/*
 * Created on Sept 21, 2007
 *
 */
package com.x8ing.mc;

import java.io.Serializable;

/**
 * Central holder for relevant properties that control the whole simulation.
 * 
 * @author Patrick Heusser
 */

public class Configuration implements Serializable {

	private static final long serialVersionUID = 2375960255535864843L;

	/**
	 * G002.
	 * the theoretical value of the process. might be reduced by bugs that are discovered.
	 */
	private double pannedProcessGain = 2100;

	/**
	 * P006.
	 * chance for a successful deployment of the software.
	 */
	private int chanceDeployAndRealeaseDeploymentSuccess = 70;

	/**
	 * P004.
	 * change that the developer fixed a bug seccessfully.
	 */
	private int chanceFixBugWithSuccess = 75;

	/**
	 * P003.
	 * chance that we discover a new bug in production.
	 */
	private int chanceForFindingANewBugInProduction = 60;

	/**
	 * P002.
	 * chance that the production support team can resolve a failure.
	 */
	private int chanceOperationDownResolvingWithSuccess = 90;

	/**
	 * chance that the production support team can resolve a failure.
	 */
	private int chanceOperationHasOutage = 20;

	/**
	 * P005.
	 * chance for testing a bug sucessfully.
	 */
	private int chanceTestingBugWithSuccess = 80;

	/**
	 * T003.
	 * defines how many business contexts are collected (due to huge memory consumption).
	 * 
	 * @see MonteCarloControler#partialCollectedBusinessContextHistory
	 */
	private int collectMaxNumberOfBusinessContext = 3;

	/**
	 * F005.
	 * cost for action: adding the bug to fix later buglist.
	 */
	private double costActionAddBugToFixList = 170;

	/**
	 * F004.
	 * cost for action: analyze bug loss.
	 */
	private double costActionAnalyzeCostLoss = 200;

	/**
	 * F003.
	 * cost for action: for production deployment.
	 */
	private double costActionDeployAndRelease = 2400;

	/**
	 * F002.
	 * cost for action: document and finish the process.
	 */
	private double costActionDocumentAndFinishProcess = 380;

	/**
	 * F001.
	 * cost for action: testing.
	 */
	private double costActionTesting = 2500;

	/**
	 * G012.
	 * for gauss distribution: analyzeCostLossAction
	 */
	private double distributionBugFixingCostMax = 500;

	/**
	 * G011.
	 * for gauss distribution: analyzeCostLossAction
	 */
	private double distributionBugFixingCostMin = 100;

	/**
	 * G010.
	 * for gauss distribution: analyzeCostLossAction
	 */
	private int distributionBugFixingCostStdDev = 2;

	/**
	 * T002.
	 * how many loops in MC will be executed.
	 */
	private int numberCompleteSimulationLoops = 1000;

	/**
	 * One simulation loop should have this duration in days.
	 */
	private int numberOfSimulationDays = 120;

	/**
	 * G004.
	 * outage is a constant cost factor per day
	 */
	private double operationCostForOutagePerDay = 1000;

	/**
	 * G003.
	 * when it's worth to start with fixing of bugs. condition value.
	 */
	private double processCostLossTriggerForBugFixing = 300;

	/**
	 * G001.
	 * This is our MAJOR number, and actually the cost caused by not nonconformity of the software. this means, how many
	 * bugs we delivered.
	 */
	private double softwareInitialNoncCnformityCost = 500;

	public int getChanceDeployAndRealeaseDeploymentSuccess() {
		return chanceDeployAndRealeaseDeploymentSuccess;
	}

	public int getChanceFixBugWithSuccess() {
		return chanceFixBugWithSuccess;
	}

	public int getChanceForFindingANewBugInProduction() {
		return chanceForFindingANewBugInProduction;
	}

	public int getChanceOperationDownResolvingWithSuccess() {
		return chanceOperationDownResolvingWithSuccess;
	}

	public int getChanceOperationHasOutage() {
		return chanceOperationHasOutage;
	}

	public int getChanceTestingBugWithSuccess() {
		return chanceTestingBugWithSuccess;
	}

	public int getCollectMaxNumberOfBusinessContext() {
		return collectMaxNumberOfBusinessContext;
	}

	public double getCostActionAddBugToFixList() {
		return costActionAddBugToFixList;
	}

	public double getCostActionAnalyzeCostLoss() {
		return costActionAnalyzeCostLoss;
	}

	public double getCostActionDeployAndRelease() {
		return costActionDeployAndRelease;
	}

	public double getCostActionDocumentAndFinishProcess() {
		return costActionDocumentAndFinishProcess;
	}

	public double getCostActionTesting() {
		return costActionTesting;
	}

	public double getDistributionBugFixingCostMax() {
		return distributionBugFixingCostMax;
	}

	public double getDistributionBugFixingCostMin() {
		return distributionBugFixingCostMin;
	}

	public int getDistributionBugFixingCostStdDev() {
		return distributionBugFixingCostStdDev;
	}

	public int getNumberCompleteSimulationLoops() {
		return numberCompleteSimulationLoops;
	}

	public double getOperationCostForOutagePerDay() {
		return operationCostForOutagePerDay;
	}

	public double getProcessCostLossTriggerForBugFixing() {
		return processCostLossTriggerForBugFixing;
	}

	public double getSoftwareInitialNoncCnformityCost() {
		return softwareInitialNoncCnformityCost;
	}

	public void setChanceDeployAndRealeaseDeploymentSuccess(int chanceDeployAndRealeaseDeploymentSuccess) {
		this.chanceDeployAndRealeaseDeploymentSuccess = chanceDeployAndRealeaseDeploymentSuccess;
	}

	public void setChanceFixBugWithSuccess(int chanceFixBugWithSuccess) {
		this.chanceFixBugWithSuccess = chanceFixBugWithSuccess;
	}

	public void setChanceForFindingANewBugInProduction(int chanceForFindingANewBugInProduction) {
		this.chanceForFindingANewBugInProduction = chanceForFindingANewBugInProduction;
	}

	public void setChanceOperationDownResolvingWithSuccess(int chanceOperationDownResolvingWithSuccess) {
		this.chanceOperationDownResolvingWithSuccess = chanceOperationDownResolvingWithSuccess;
	}

	public void setChanceOperationHasOutage(int chanceOperationHasOutage) {
		this.chanceOperationHasOutage = chanceOperationHasOutage;
	}

	public void setChanceTestingBugWithSuccess(int chanceTestingBugWithSuccess) {
		this.chanceTestingBugWithSuccess = chanceTestingBugWithSuccess;
	}

	public void setCollectMaxNumberOfBusinessContext(int collectMaxNumberOfBusinessContext) {
		this.collectMaxNumberOfBusinessContext = collectMaxNumberOfBusinessContext;
	}

	public void setCostActionAddBugToFixList(double costActionAddBugToFixList) {
		this.costActionAddBugToFixList = costActionAddBugToFixList;
	}

	public void setCostActionAnalyzeCostLoss(double costActionAnalyzeCostLossAction) {
		this.costActionAnalyzeCostLoss = costActionAnalyzeCostLossAction;
	}

	public void setCostActionDeployAndRelease(double costActionDeployAndRelease) {
		this.costActionDeployAndRelease = costActionDeployAndRelease;
	}

	public void setCostActionDocumentAndFinishProcess(double costActionDocumentAndFinishProcess) {
		this.costActionDocumentAndFinishProcess = costActionDocumentAndFinishProcess;
	}

	public void setCostActionTesting(double costActionTesting) {
		this.costActionTesting = costActionTesting;
	}

	public void setDistributionBugFixingCostMax(double distributionBugFixingCostMax) {
		this.distributionBugFixingCostMax = distributionBugFixingCostMax;
	}

	public void setDistributionBugFixingCostMin(double distributionBugFixingCostMin) {
		this.distributionBugFixingCostMin = distributionBugFixingCostMin;
	}

	public void setDistributionBugFixingCostStdDev(int distributionBugFixingCostStdDev) {
		this.distributionBugFixingCostStdDev = distributionBugFixingCostStdDev;
	}

	public void setNumberCompleteSimulationLoops(int numberCompleteSimulationLoops) {
		this.numberCompleteSimulationLoops = numberCompleteSimulationLoops;
	}

	public void setOperationCostForOutagePerDay(double operationCostForOutagePerDay) {
		this.operationCostForOutagePerDay = operationCostForOutagePerDay;
	}

	public void setProcessCostLossTriggerForBugFixing(double processCostLossTriggerForBugFixing) {
		this.processCostLossTriggerForBugFixing = processCostLossTriggerForBugFixing;
	}

	public void setSoftwareInitialNoncCnformityCost(double softwareInitialNoncCnformityCost) {
		this.softwareInitialNoncCnformityCost = softwareInitialNoncCnformityCost;
	}

	public int getNumberOfSimulationDays() {
		return numberOfSimulationDays;
	}

	public void setNumberOfSimulationDays(int numberOfSimulationDays) {
		this.numberOfSimulationDays = numberOfSimulationDays;
	}

	public double getPannedProcessGain() {
		return pannedProcessGain;
	}

	public void setPannedProcessGain(double pannedProcessGain) {
		this.pannedProcessGain = pannedProcessGain;
	}

	public String toString() {

		StringBuffer sb = new StringBuffer();

		sb.append(" pannedProcessGain=");
		sb.append(pannedProcessGain);

		sb.append(" chanceDeployAndRealeaseDeploymentSuccess=");
		sb.append(chanceDeployAndRealeaseDeploymentSuccess);

		sb.append(" chanceFixBugWithSuccess=");
		sb.append(chanceFixBugWithSuccess);

		sb.append(" chanceForFindingANewBugInProduction=");
		sb.append(chanceForFindingANewBugInProduction);

		sb.append(" chanceOperationHasOutage=");
		sb.append(chanceOperationHasOutage);

		sb.append(" chanceTestingBugWithSuccess=");
		sb.append(chanceTestingBugWithSuccess);

		sb.append(" collectMaxNumberOfBusinessContext=");
		sb.append(collectMaxNumberOfBusinessContext);

		sb.append(" costActionAddBugToFixList=");
		sb.append(costActionAddBugToFixList);

		sb.append(" costActionAnalyzeCostLoss=");
		sb.append(costActionAnalyzeCostLoss);

		sb.append(" costActionDeployAndRelease=");
		sb.append(costActionDeployAndRelease);

		sb.append(" costActionDocumentAndFinishProcess=");
		sb.append(costActionDocumentAndFinishProcess);

		sb.append(" costActionTesting=");
		sb.append(costActionTesting);

		sb.append(" distributionBugFixingCostMax=");
		sb.append(distributionBugFixingCostMax);

		sb.append(" distributionBugFixingCostMin=");
		sb.append(distributionBugFixingCostMin);

		sb.append(" distributionBugFixingCostStdDev=");
		sb.append(distributionBugFixingCostStdDev);

		sb.append(" numberCompleteSimulationLoops=");
		sb.append(numberCompleteSimulationLoops);

		sb.append(" numberOfSimulationDays=");
		sb.append(numberOfSimulationDays);

		sb.append(" operationCostForOutagePerDay=");
		sb.append(operationCostForOutagePerDay);

		sb.append(" processCostLossTriggerForBugFixing=");
		sb.append(processCostLossTriggerForBugFixing);

		sb.append(" softwareInitialNoncCnformityCost=");
		sb.append(softwareInitialNoncCnformityCost);

		return sb.toString();

	}

}
