/*
 * Created on Aug 30, 2007
 *
 */
package com.x8ing.mc;

import java.io.PrintStream;

/**
 * Simple start up in java standalone mode. Mostly for testing only!
 * 
 * @author Patrick Heusser
 */
public class Main {

	final static String SEP_COLUMN = "\t";

	final static String SEP_ROW = "\n";

	public static void main(String[] args) {

		if (args == null || args.length == 0) {

			simpleRun();

		} else if ("1".equals(args[0])) {

			resultRunChangeSoftwareNonConformityCostAndTriggerValue(System.out);

		} else if ("2".equals(args[0])) {

			resultRunSingleDeepRun(System.out);

		} else {

			System.err.println("command not found. taking default...");
			System.out.println("\n\n");

			// do all result gatherings
			resultRunChangeSoftwareNonConformityCostAndTriggerValue(System.out);
			resultRunSingleDeepRun(System.out);

		}

		System.out.println("\n\n\n DONE. End of all runs.");

	}

	private static void simpleRun() {
		System.out.println("START: simpleRun() ");

		Configuration configuration = new Configuration();

		MonteCarloControler mcc = new MonteCarloControler(configuration);
		mcc.runBusinessProcessSimulation();

		System.out.println(mcc.printDistribiton(100));
	}

	private static void resultRunSingleDeepRun(PrintStream ret) {

		long t1 = System.currentTimeMillis();

		final int numberOfSimulationDays = 180;
		final int numberCompleteSimulationLoops = 10000;
		final double softwareNonConfCost = 400;

		ret.print(SEP_ROW + SEP_ROW + SEP_ROW + "------------------------------------------------------");
		ret.print(SEP_ROW + "START: resultRunSingleDeepRun() ");
		ret.print(SEP_ROW + "------------------------------------------------------");
		ret.print(SEP_ROW + "numberOfSimulationDays=" + numberOfSimulationDays);
		ret.print(SEP_ROW + "numberCompleteSimulationLoops=" + numberCompleteSimulationLoops);
		ret.print(SEP_ROW + "softwareNonConfCost =" + softwareNonConfCost);
		ret.print(SEP_ROW + "------------------------------------------------------");

		Configuration configuration = new Configuration();

		configuration.setNumberOfSimulationDays(numberOfSimulationDays);
		configuration.setSoftwareInitialNoncCnformityCost(softwareNonConfCost);
		configuration.setNumberCompleteSimulationLoops(numberCompleteSimulationLoops);

		MonteCarloControler mcc = new MonteCarloControler(configuration);
		mcc.runBusinessProcessSimulation();

		ret.print(mcc.printDistribiton(100));

		ret.print(SEP_ROW + configuration);

		long dt = System.currentTimeMillis() - t1;
		ret.print("\n\ntask done... time[ms]=" + dt + "  time[min]=" + (dt / 1000 / 60));

	}

	/**
	 * change two params and display as table:
	 * 
	 * changing values: softwareNonConfCost and processCostLossTrigger
	 */
	private static String resultRunChangeSoftwareNonConformityCostAndTriggerValue(PrintStream ret) {

		long t1 = System.currentTimeMillis();

		final int numberOfSimulationDays = 180;
		final int numberCompleteSimulationLoops = 10000;

		final int softwareNonConfCostFrom = 100;
		final int softwareNonConfCostTo = 2000;
		final int softwareNonConfCostStep = 50;

		final int processCostLossTriggerFrom = 100;
		final int processCostLossTriggerTo = 1200;
		final int processCostLossTriggerStep = 100;

		ret.print(SEP_ROW + SEP_ROW + SEP_ROW + "------------------------------------------------------");
		ret.print(SEP_ROW + "START: resultRunChangeSoftwareNonConformityCost() ");
		ret.print(SEP_ROW + "------------------------------------------------------");
		ret.print(SEP_ROW + "numberOfSimulationDays=" + numberOfSimulationDays);
		ret.print(SEP_ROW + "numberCompleteSimulationLoops=" + numberCompleteSimulationLoops);
		ret.print(SEP_ROW + "------------------------------------------------------");

		// draw header
		ret.print(SEP_ROW + "softwareNonConfCost");
		ret.print(SEP_COLUMN);
		for (int processCostLossTrigger = processCostLossTriggerFrom; processCostLossTrigger <= processCostLossTriggerTo; processCostLossTrigger += processCostLossTriggerStep) {
			ret.print("processCostLossTrigger=" + processCostLossTrigger);
			ret.print(SEP_COLUMN);
		}

		Configuration configuration = null;

		// draw table
		for (int softwareNonConfCost = softwareNonConfCostFrom; softwareNonConfCost <= softwareNonConfCostTo; softwareNonConfCost += softwareNonConfCostStep) {

			ret.print(SEP_ROW);
			ret.print(String.valueOf(softwareNonConfCost));
			ret.print(SEP_COLUMN);

			for (int processCostLossTrigger = processCostLossTriggerFrom; processCostLossTrigger <= processCostLossTriggerTo; processCostLossTrigger += processCostLossTriggerStep) {

				configuration = new Configuration();

				// debug
				/*
				 * ret.print("(softwareNonConfCost=" + softwareNonConfCost + ", processCostLossTrigger=" +
				 * processCostLossTrigger + ") ");
				 */

				configuration.setNumberOfSimulationDays(numberOfSimulationDays);
				configuration.setSoftwareInitialNoncCnformityCost(softwareNonConfCost);
				configuration.setNumberCompleteSimulationLoops(numberCompleteSimulationLoops);
				configuration.setProcessCostLossTriggerForBugFixing(processCostLossTrigger);

				MonteCarloControler mcc = new MonteCarloControler(configuration);
				mcc.runBusinessProcessSimulation();

				Statistic statistic = new Statistic(mcc.getBalanceValues());

				ret.print(statistic.getAverage() + SEP_COLUMN);

			}

		}

		// print one last config for an idea about the other params
		ret.print(SEP_ROW + SEP_ROW + configuration);

		long dt = System.currentTimeMillis() - t1;
		ret.print("\n\ntask done... time[ms]=" + dt + "  time[min]=" + (dt / 1000 / 60));

		return ret.toString();
	}

}
