/*
 * Created on Aug 31, 2007
 */
package com.x8ing.mc;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.x8ing.mc.bp.BusinessContext;
import com.x8ing.mc.bp.BusinessProcesses;

/**
 * Starts the whole simulation for a defined number of times and collects the data.
 * 
 * @author Patrick Heusser
 */
public class MonteCarloControler implements Serializable {

	private static final long serialVersionUID = -5686731204223731697L;

	private Configuration configuration = null;

	private BusinessProcesses businessProcesses = null;

	/**
	 * We CAN NOT collect all business contexts, since the memory footprint is to heavy. so, we collect only a certain
	 * amount, e.g. each 100th. the collection rate is controled by the variable collectEveryNthBusinessContext.
	 * <p>
	 * therefore this collection is only a subset of all contexts.
	 * 
	 * type: BusinessContext
	 * 
	 */
	private List partialCollectedBusinessContextHistory = null;

	/**
	 * here we store all balance Values. we can not use all business contexts since huge memory consumption
	 * 
	 * @see #partialCollectedBusinessContextHistory
	 */
	private double balanceValues[] = null;

	public MonteCarloControler(Configuration configuration) {
		this.configuration = configuration;
	}

	public void runBusinessProcessSimulation() {

		init();

		for (int loop = 0; loop < configuration.getNumberCompleteSimulationLoops(); loop++) {

			BusinessContext businessContext = businessProcesses.run();

			// check if we should collect this business context
			if (loop <= configuration.getCollectMaxNumberOfBusinessContext()) {
				partialCollectedBusinessContextHistory.add(businessContext);

			}
			balanceValues[loop] = businessContext.getBalanceSheet().calculateCurrentBalance();

		}

		businessProcesses.run();

	}

	public double[] getBalanceValues() {

		return balanceValues;
	}

	public String printDistribiton(int numberOfRiskSegments) {

		Statistic statistic = new Statistic(getBalanceValues());
		return "\n\n" + statistic.getDistributionPrint(numberOfRiskSegments) + " \naverage=" + statistic.getAverage();

	}

	private void init() {

		partialCollectedBusinessContextHistory = new ArrayList();
		balanceValues = new double[configuration.getNumberCompleteSimulationLoops()];

		businessProcesses = new BusinessProcesses(configuration);

	}

	public List getPartialCollectedBusinessContextHistory() {
		return partialCollectedBusinessContextHistory;
	}

	/**
	 * for bean access
	 */
	public String getPrintDistribiton() {
		return printDistribiton(100);
	}

	/**
	 * for bean access
	 */
	public String getBalanceValuesPrint() {

		StringBuffer sb = new StringBuffer();
		sb.append("iterationNumber \tearning\n");

		for (int i = 0; i < balanceValues.length; i++) {
			double v = balanceValues[i];
			sb.append(i + 1);
			sb.append("\t");
			sb.append(v);
			sb.append("\n");
		}

		return sb.toString();

	}
}
