/*
 * Created on Aug 31, 2007
 *
 */
package com.x8ing.mc;

/**
 * @author Patrick Heusser
 */
public class Statistic {

	private double[] values = null;

	private Statistic() {

	}

	public Statistic(double values[]) {
		this.values = values;
	}

	public ResultDistributionEntry[] getDistribution(int numberOfSegments) {

		double[] extrema = getMinimumAndMaximum();
		double spread = extrema[1] - extrema[0];

		double slotWidth = spread / (numberOfSegments - 1);

		ResultDistributionEntry entries[] = new ResultDistributionEntry[numberOfSegments];

		// init array
		for (int i = 0; i < numberOfSegments; i++) {

			ResultDistributionEntry resultDistributionEntry = new ResultDistributionEntry();
			entries[i] = resultDistributionEntry;

			resultDistributionEntry.numberOfEntries = 0;
			resultDistributionEntry.rangeFrom = extrema[0] + i * slotWidth;
			resultDistributionEntry.rangeTo = extrema[0] + (i + 1) * slotWidth;

		}

		// count entries
		for (int i = 0; i < values.length; i++) {
			double value = values[i];

			int entrySlot = (int) ((value - extrema[0]) / slotWidth);

			ResultDistributionEntry entry = entries[entrySlot];

			entry.numberOfEntries++;
			entry.percentile = ((double) entry.numberOfEntries) * 100 / values.length;

		}

		return entries;
	}

	public String getDistributionPrint(int numberOfSegments) {

		StringBuffer sb = new StringBuffer();

		ResultDistributionEntry[] entries = getDistribution(numberOfSegments);

		// write header
		sb.append("rangeFrom \trangeTo \tcount \tpercentil \n");

		// write entries
		for (int i = 0; i < entries.length; i++) {
			ResultDistributionEntry entry = entries[i];
			sb.append(entry.rangeFrom).append("\t").append(entry.rangeTo).append("\t").append(entry.numberOfEntries).append("\t")
					.append(entry.percentile).append("\n");
		}

		return sb.toString();

	}

	public double getAverage() {

		double sum = 0;

		for (int i = 0; i < values.length; i++) {
			sum += values[i];

		}

		return sum / values.length;
	}

	/**
	 * 
	 * @return an double array: double[0] is minimum, double[1] is maximum.
	 */
	public double[] getMinimumAndMaximum() {

		double[] extrema = new double[2];

		extrema[0] = Double.MAX_VALUE;
		extrema[1] = Double.MIN_VALUE;

		for (int i = 0; i < values.length; i++) {

			double v = values[i];

			// check for new minima
			if (v < extrema[0]) {
				extrema[0] = v;
			}

			// check for new maxima
			if (v > extrema[1]) {
				extrema[1] = v;
			}

		}

		return extrema;

	}

	public static class ResultDistributionEntry {

		public double rangeFrom = 0;

		public double rangeTo = 0;

		public int numberOfEntries = 0;

		public double percentile = 0;

	}

}
