/*
 * Created on Oct 4, 2007
 */
package com.x8ing.mc.bp;

/**
 * A money transaction must belong to a certain account. This allows detailed financial analysis where the money has
 * been spend or gained respectively.
 * 
 * Use the static final class members as accounts.
 * 
 * @author Patrick Heusser
 */
public class BalanceAccount {

	private String name = null;

	private String description = null;

	private BalanceAccount(String name, String description) {
		this.name = name;
		this.description = description;
	}

	/**
	 * keeps track of production process gain, and outages
	 */
	public static final BalanceAccount PRODUCTION = new BalanceAccount("Production",
			"keeps track of production process gain an outages");

	/**
	 * bug analyzing costs
	 */
	public static final BalanceAccount BUG_ANALYZING = new BalanceAccount("BugAnalyzing", "keeps track of bug analyzing costs");

	/**
	 * develop and fix later costs
	 */
	public static final BalanceAccount DEVELOP = new BalanceAccount("Development", "keeps track of fixing and fix-later cost");

	/**
	 * testing cost
	 */
	// SPACE at end for formatting
	public static final BalanceAccount TESTING = new BalanceAccount("Testing ", "keeps track of testing cost");

	/**
	 * deployment, release and documentation cost.
	 */
	public static final BalanceAccount DEPLOYMENT = new BalanceAccount("Deployment",
			"keeps track of deployment and releasing cost.");

	public boolean equals(Object obj) {

		if (obj == null || !(obj instanceof BalanceAccount)) {
			return false;
		}

		BalanceAccount account2 = (BalanceAccount) obj;

		return account2.getName().equals(this.getName());

	}

	public int hashCode() {

		return getName().hashCode();

	}

	public String toString() {

		return "BalanceAccount: name=" + name + " description=" + description;
	}

	public String getDescription() {
		return description;
	}

	public String getName() {
		return name;
	}
}
