/*
 * Created on Aug 29, 2007
 */
package com.x8ing.mc.bp;

import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * Takes care of all Moneytransactions and calculates the balance (saldo).
 * 
 * @author Patrick Heusser
 */
public class BalanceSheet {

	private SimpleDateFormat simpleDateFormat = new SimpleDateFormat(Constants.GLOBAL_DATE_FORMAT);

	/**
	 * keeps a set of all accounts ever used.
	 */
	private Set currentUsedAccounts = new HashSet();

	private NumberFormat numberFormat = Constants.createCurrencyFormater();

	/**
	 * type: MoneyTransaction
	 */
	private List moneyTransactions = new ArrayList();

	public void addTransaction(MoneyTransaction moneyTransaction) {

		// add account to accountSet
		currentUsedAccounts.add(moneyTransaction.getBalanceAccount());

		moneyTransactions.add(moneyTransaction);

		if (Constants.TRACE_DEBUG) {

			System.out.println(simpleDateFormat.format(moneyTransaction.getTransactionDate().getTime()) + "\tBalanceSheet:\t"
					+ moneyTransaction.getAmount() + "\t" + moneyTransaction.getBookingDescription() + "\tcurrentBalance="
					+ numberFormat.format(calculateCurrentBalance()));
		}

	}

	public void addTransaction(String bookingDescription, double amount, Calendar date, BalanceAccount account) {
		addTransaction(new MoneyTransaction(bookingDescription, date, amount, account));
	}

	/**
	 * 
	 * @param accounts
	 * @return List with type MoneyTransaction
	 */
	public List searchMoneyTransactionOfAccountTypes(BalanceAccount[] accounts) {

		List ret = new ArrayList();

		if (accounts == null || accounts.length == 0) {
			return ret;
		}

		Collection c = Arrays.asList(accounts);
		Set accountSet = new HashSet(c);

		for (Iterator it = moneyTransactions.iterator(); it.hasNext();) {
			MoneyTransaction moneyTransaction = (MoneyTransaction) it.next();

			if (accountSet.contains(moneyTransaction.getBalanceAccount())) {
				ret.add(moneyTransaction);
			}
		}

		return ret;

	}

	/**
	 * calculates the balance only for the passed accounts
	 */
	public double calculateCurrentBalanceForAccounts(BalanceAccount[] accounts) {

		List subset = searchMoneyTransactionOfAccountTypes(accounts);

		return calculateCurrentBalanceImpl(subset);

	}

	/**
	 * calculate grand overal balance.
	 */
	public double calculateCurrentBalance() {

		return calculateCurrentBalanceImpl(moneyTransactions);

	}

	private static double calculateCurrentBalanceImpl(List listOfMoneyTransactions) {

		double sum = 0;

		for (Iterator it = listOfMoneyTransactions.iterator(); it.hasNext();) {
			MoneyTransaction moneyTransaction = (MoneyTransaction) it.next();
			sum += moneyTransaction.getAmount();
		}

		return sum;

	}

	public BalanceAccount[] getAllUsedAccounts() {
		return (BalanceAccount[]) currentUsedAccounts.toArray(new BalanceAccount[0]);
	}

	public String printBalanceSheetAccountOverview() {
		return printBalanceSheetAccountOverview(getAllUsedAccounts());
	}

	public String printBalanceSheetAccountOverview(BalanceAccount[] accounts) {

		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < accounts.length; i++) {

			BalanceAccount account = accounts[i];

			sb.append("Account: \t");
			sb.append(account.getName());
			sb.append("\t");
			sb.append(numberFormat.format(calculateCurrentBalanceForAccounts(new BalanceAccount[] { account })));
			
			sb.append("\n");
		}

		sb.append("\n\t\tgrand total:\t" + numberFormat.format(calculateCurrentBalance()));
		sb.append("\n");

		return sb.toString();

	}

	public String printBalanceSheetByAccount(boolean supressSameDates) {
		return printBalanceSheetByAccount(supressSameDates, getAllUsedAccounts());
	}

	public String printBalanceSheetByAccount(boolean supressSameDates, BalanceAccount[] accounts) {

		StringBuffer sb = new StringBuffer();
		final String SEP = ",";

		for (int i = 0; i < accounts.length; i++) {

			BalanceAccount account = accounts[i];

			List subsetMoneyTransactions = searchMoneyTransactionOfAccountTypes(new BalanceAccount[] { account });

			sb.append("Account: " + account.getName() + "  (" + account.getDescription() + ")");
			sb.append("\n");

			String lastPrintedDate = "";

			for (Iterator it = subsetMoneyTransactions.iterator(); it.hasNext();) {

				sb.append("\t");

				MoneyTransaction moneyTransaction = (MoneyTransaction) it.next();

				String currentDate = simpleDateFormat.format(moneyTransaction.getTransactionDate().getTime());

				if (supressSameDates && currentDate.equals(lastPrintedDate)) {
					sb.append("          ");
				} else {
					sb.append(currentDate);
				}

				sb.append(SEP).append("\t");

				sb.append(numberFormat.format(moneyTransaction.getAmount()));
				sb.append(SEP).append("\t");

				sb.append(moneyTransaction.getBookingDescription());

				sb.append("\n");

			}

			sb.append("Account total =\t");
			sb.append(numberFormat.format(calculateCurrentBalanceForAccounts(new BalanceAccount[] { account })));
			sb.append("\n\n\n");

		}

		sb.append("GRAND TOTAL OVER ALL ACCOUNTS: " + numberFormat.format(calculateCurrentBalance()));
		sb.append("\n");

		return sb.toString();

	}

	public String printBalanceSheetByDate(boolean supressSameDates) {

		StringBuffer sb = new StringBuffer();

		final String SEP = ",";

		sb.append("date").append(SEP).append("\tamount").append(SEP).append("\taccount \tdescription \n\n");

		String lastPrintedDate = "";

		for (Iterator it = moneyTransactions.iterator(); it.hasNext();) {

			MoneyTransaction moneyTransaction = (MoneyTransaction) it.next();

			String currentDate = simpleDateFormat.format(moneyTransaction.getTransactionDate().getTime());

			if (supressSameDates && currentDate.equals(lastPrintedDate)) {
				sb.append("          ");
			} else {
				sb.append(currentDate);
			}
			sb.append(SEP).append("\t");

			sb.append(moneyTransaction.getBalanceAccount().getName());
			sb.append(SEP).append("\t");

			sb.append(numberFormat.format(moneyTransaction.getAmount()));
			sb.append(SEP).append("\t");

			sb.append(moneyTransaction.getBookingDescription());
			sb.append(SEP).append("\t");

			sb.append("\n");

		}

		sb.append("\n\nBalance grand total = ").append(numberFormat.format(calculateCurrentBalance()));

		return sb.toString();

	}

	public String toString() {
		return printBalanceSheetByAccount(false, getAllUsedAccounts());
	}

	/**
	 * for bean access.
	 */
	public String getPrintBalanceSheetByDate() {

		return printBalanceSheetByDate(true);

	}

	/**
	 * for bean access.
	 */
	public String getPrintBalanceSheetByAccount() {

		BalanceAccount[] accounts = getAllUsedAccounts();
		return printBalanceSheetByAccount(true, accounts);

	}
	
	/**
	 * for bean access.
	 */
	public String getPrintBalanceSheetAccountOverview() {

		return printBalanceSheetAccountOverview();

	}
	

}