/*
 * Created on Sep 4, 2007
 */
package com.x8ing.mc.bp;

import java.util.Calendar;

/**
 * Data object to store the characteristics of a bug.
 * 
 * @author Patrick Heusser
 */
public class Bug {

	private int bugID = 0;

	private BugState bugState = null;

	/**
	 * the date we found the bug.
	 */
	private Calendar discoveryDate = null;

	/**
	 * what's is the impact on the business process and the loss of profitability.
	 */
	private double negativImpactOnBusinessProcess = 0;

	/**
	 * what are the cost to fix the bug.
	 */
	private double estimatedCostToFixBug = 0;

	/**
	 * @param discoveryDate
	 * @param negativImpactOnBusinessProcess
	 * @param estimatedCostToFixBug
	 * @param bugID
	 *            will be given by BugList.
	 */
	public Bug(Calendar discoveryDate, double negativImpactOnBusinessProcess, double estimatedCostToFixBug, int bugID) {
		super();
		this.discoveryDate = discoveryDate;
		this.negativImpactOnBusinessProcess = negativImpactOnBusinessProcess;
		this.estimatedCostToFixBug = estimatedCostToFixBug;
		this.bugID = bugID;
		this.bugState = Bug.BugState.STATE_BUG_NEW;
	}

	private Bug() {

	}

	public Calendar getDiscoveryDate() {
		return discoveryDate;
	}

	public void setDiscoveryDate(Calendar discoveryDate) {
		this.discoveryDate = discoveryDate;
	}

	public double getEstimatedCostToFixBug() {
		return estimatedCostToFixBug;
	}

	public void setEstimatedCostToFixBug(double estimatedCostToFixBug) {
		this.estimatedCostToFixBug = estimatedCostToFixBug;
	}

	public double getNegativImpactOnBusinessProcess() {
		return negativImpactOnBusinessProcess;
	}

	public void setNegativImpactOnBusinessProcess(double negativImpaceOnBusinessProcess) {
		this.negativImpactOnBusinessProcess = negativImpaceOnBusinessProcess;
	}

	public int getBugID() {
		return bugID;
	}

	/**
	 * kind of an enum.
	 * 
	 * @author Patrick Heusser
	 * 
	 */
	public static class BugState {

		private String bugState = null;

		private BugState(String state) {
			this.bugState = state;
		}

		public static final BugState STATE_BUG_NEW = new BugState("new");

		public static final BugState STATE_BUG_FIX_LATER = new BugState("fixLater");

		public static final BugState STATE_BUG_FIXED = new BugState("fixed");

		public static final BugState STATE_BUG_DEPLOYED = new BugState("deployed");

		public static final BugState STATE_BUG_TEST_FAILED = new BugState("testFailed");

		public static final BugState STATE_BUG_TEST_SUCESS = new BugState("testSuccess");

		public boolean equals(Object obj) {

			if (obj == null || !(obj instanceof BugState)) {
				return false;
			}

			BugState state2 = (BugState) obj;

			return state2.bugState.equals(this.bugState);

		}

		public int hashCode() {
			return bugState.hashCode();
		}

		public String toString() {
			return bugState;
		}

	}

	public BugState getBugState() {
		return bugState;
	}

	public void setBugState(BugState bugState) {
		this.bugState = bugState;
	}

	public String toString() {
		return "bugID=" + bugID + " bugState=" + bugState;
	}
}
