/*
 * Created on Aug 29, 2007
 *
 */
package com.x8ing.mc.bp;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import com.x8ing.lsm4j.StateContext;
import com.x8ing.mc.Configuration;

/**
 * @author Patrick Heusser
 * 
 */
public class BusinessContext implements StateContext {

	private Configuration configuration = null;

	private BalanceSheet balanceSheet = new BalanceSheet();

	private LogBook logBook = new LogBook();

	private boolean productionSystemRunning = true;

	/**
	 * if the changeProcess is already running we queue the bugs and wait<br>
	 * type: Bug
	 */
	private List queuedBugs = new ArrayList();

	/**
	 * is the develop department already doing something?
	 */
	private boolean developProcessBusy = false;

	private boolean deployedWithSucess = true;

	/**
	 * bugs the develop (change) process is operating on.
	 */
	private BugList bugs = new BugList();

	/**
	 * all losses that are caused through remaining bugs in software. this value changes if a deployment is done!
	 */
	private double processLossCausedByBugsCurrentValue = 0;

	/**
	 * the simulation date
	 */
	private Calendar currentDate = GregorianCalendar.getInstance();

	private boolean noMoreDiscoveringOfBugs = false;

	public BusinessContext(Configuration configuration) {

		this.configuration = configuration;

		init();
	}

	private void init() {

		processLossCausedByBugsCurrentValue = configuration.getSoftwareInitialNoncCnformityCost();

	}

	public void transferQueuedBugsToDevelopBugList() {

		for (Iterator it = queuedBugs.iterator(); it.hasNext();) {
			Bug bug = (Bug) it.next();
			bugs.addBug(bug);
			getLogBook().addLogEntry(getCurrentDate(), "transfered bug to developers. bugID=" + bug.getBugID());

			// remove from queues list
			it.remove();
		}

	}

	public double calculateTotalProcessLossOfAllBugsEverFound() {

		final Bug.BugState[] ALL_STATES = new Bug.BugState[] { Bug.BugState.STATE_BUG_DEPLOYED, Bug.BugState.STATE_BUG_FIX_LATER,
				Bug.BugState.STATE_BUG_FIXED, Bug.BugState.STATE_BUG_NEW, Bug.BugState.STATE_BUG_TEST_FAILED,
				Bug.BugState.STATE_BUG_TEST_SUCESS

		};

		return calculateTotalProcessLoss(ALL_STATES);

	}

	/**
	 * 
	 * @return the total process losscaused by all bugs that could be fixed but are not yet.
	 */
	public double calculateTotalProcessLossOfAllBugsNotDeployedYet() {

		final Bug.BugState[] STATES_NOT_FIXED_YET = new Bug.BugState[] { Bug.BugState.STATE_BUG_NEW,
				Bug.BugState.STATE_BUG_FIX_LATER, Bug.BugState.STATE_BUG_TEST_FAILED, Bug.BugState.STATE_BUG_FIXED };

		return calculateTotalProcessLoss(STATES_NOT_FIXED_YET);

	}

	private double calculateTotalProcessLoss(final Bug.BugState[] states) {

		List allBugs = getBugs().getBugsWithState(states);

		double totalLoss = 0;
		for (Iterator iter = allBugs.iterator(); iter.hasNext();) {
			Bug bug = (Bug) iter.next();
			totalLoss += bug.getNegativImpactOnBusinessProcess();
		}

		return totalLoss;
	}

	/**
	 * what is the current value of the process.
	 */
	public double getCurrentProcessGain() {
		return configuration.getPannedProcessGain() - getProcessLossCausedByBugsCurrentValue();
	}

	/**
	 * @return Returns the balanceSheet.
	 */
	public BalanceSheet getBalanceSheet() {
		return balanceSheet;
	}

	/**
	 * @return Returns the currentDate.
	 */
	public Calendar getCurrentDate() {
		return currentDate;
	}

	/**
	 * @param currentDate
	 *            The currentDate to set.
	 */
	public void setCurrentDate(GregorianCalendar currentDate) {
		this.currentDate = currentDate;
	}

	/**
	 * @return Returns the productionSystemRunning.
	 */
	public boolean isProductionSystemRunning() {
		return productionSystemRunning;
	}

	/**
	 * @return Returns the logBook.
	 */
	public LogBook getLogBook() {
		return logBook;
	}

	public void setProductionSystemRunning(boolean productionSystemRunning) {
		this.productionSystemRunning = productionSystemRunning;
	}

	public BugList getBugs() {
		return bugs;
	}

	public void setBugs(BugList bugs) {
		this.bugs = bugs;
	}

	public boolean isDeployedWithSucess() {
		return deployedWithSucess;
	}

	public void setDeployedWithSucess(boolean deployedWithSucess) {
		this.deployedWithSucess = deployedWithSucess;
	}

	public boolean isDevelopProcessBusy() {
		return developProcessBusy;
	}

	public void setDevelopProcessBusy(boolean developProcessBusy) {
		this.developProcessBusy = developProcessBusy;
	}

	public List getQueuedBugs() {
		return queuedBugs;
	}

	public void setQueuedBugs(List queuedBugs) {
		this.queuedBugs = queuedBugs;
	}

	public double getProcessLossCausedByBugsCurrentValue() {
		return processLossCausedByBugsCurrentValue;
	}

	public void setProcessLossCausedByBugsCurrentValue(double processLossCausedByBugsCurrentValue) {
		this.processLossCausedByBugsCurrentValue = processLossCausedByBugsCurrentValue;
	}

	public Configuration getConfiguration() {
		return configuration;
	}

	public boolean isNoMoreDiscoveringOfBugs() {
		return noMoreDiscoveringOfBugs;
	}

	public void setNoMoreDiscoveringOfBugs(boolean noMoreDiscoveringOfBugs) {
		this.noMoreDiscoveringOfBugs = noMoreDiscoveringOfBugs;
	}

}
