/*
 * Created on Aug 29, 2007
 */
package com.x8ing.mc.bp.develop;

import java.util.Iterator;
import java.util.List;

import com.x8ing.lsm4j.Condition;
import com.x8ing.lsm4j.state.ProcessableState;
import com.x8ing.mc.bp.AbstractBusinessAction;
import com.x8ing.mc.bp.BalanceAccount;
import com.x8ing.mc.bp.Bug;
import com.x8ing.mc.bp.BusinessContext;
import com.x8ing.mc.distribution.RandomDistribution;
import com.x8ing.mc.distribution.RandomDistributionGauss;

/**
 * @author Patrick Heusser
 */
public class AnalyzeCostLossAction extends AbstractBusinessAction {

	private RandomDistributionGauss bugProcessValueLoss = null;

	private RandomDistribution bugFixingCosts = null;

	/**
	 * @see com.x8ing.mc.bp.AbstractBusinessAction#execute(com.x8ing.lsm4j.state.ProcessableState,
	 *      com.x8ing.mc.bp.BusinessContext, com.x8ing.lsm4j.Condition, java.util.List)
	 */
	public void execute(ProcessableState currentState, BusinessContext businessContext, Condition previousCondition,
			List lastVisitedStatesHistory) {

		// mark that we are start doing something
		businessContext.setDevelopProcessBusy(true);

		// TODO: FIX BUG HERE!!!!!!!!!!!

		// adjust process loss probability for having process improvement
		// TODO: document this in thesis work! there are much more sophisticated ways to
		// approximate the bug finding curve.
		int rangeMin = 0;
		int rangeMax = (int) ((businessContext.getConfiguration().getSoftwareInitialNoncCnformityCost() - businessContext
				.calculateTotalProcessLossOfAllBugsEverFound()) / 12);

		if (rangeMin >= rangeMax) {
			rangeMax = rangeMin + 1;
		}

		if (rangeMax < 3) {
			// software is almost perfect
			businessContext.setNoMoreDiscoveringOfBugs(true);
			addLogBookEntry("no more analyzing and discovering of bugs possible");

		}

		bugProcessValueLoss.init(rangeMin, rangeMax, 2);

		// find recently added bugs and evaluate them
		for (Iterator bugIt = businessContext.getBugs().getBugsWithStateNew().iterator(); bugIt.hasNext();) {

			Bug recentlyDiscoveredBug = (Bug) bugIt.next();

			double processLossDueToBug = Math.round(bugProcessValueLoss.getNextRandomNumber());
			double bugFixingCost = Math.round(bugFixingCosts.getNextRandomNumber());

			// set numbers that we discovered
			recentlyDiscoveredBug.setEstimatedCostToFixBug(bugFixingCost);
			recentlyDiscoveredBug.setNegativImpactOnBusinessProcess(processLossDueToBug);

			addBalanceSheetTransaction("Analyzing of cost loss", -businessContext.getConfiguration()
					.getCostActionAnalyzeCostLoss(), BalanceAccount.BUG_ANALYZING);

			addLogBookEntry("Analyzing cost loss. Bug (id=" + recentlyDiscoveredBug.getBugID() + ") decreases process value by "
					+ processLossDueToBug);
		}

	}

	protected void lazyInit() {

		bugProcessValueLoss = new RandomDistributionGauss(-1, 1, 1);
		bugFixingCosts = new RandomDistributionGauss(businessContext.getConfiguration().getDistributionBugFixingCostMin(),
				businessContext.getConfiguration().getDistributionBugFixingCostMax(), businessContext.getConfiguration()
						.getDistributionBugFixingCostStdDev());

	}

}
