/*
 * Created on Aug 29, 2007
 */
package com.x8ing.mc.bp.develop;

import java.util.Iterator;
import java.util.List;

import com.x8ing.lsm4j.Condition;
import com.x8ing.lsm4j.state.ProcessableState;
import com.x8ing.mc.bp.AbstractBusinessAction;
import com.x8ing.mc.bp.BalanceAccount;
import com.x8ing.mc.bp.Bug;
import com.x8ing.mc.bp.BusinessContext;
import com.x8ing.mc.distribution.RandomDistribution;
import com.x8ing.mc.distribution.RandomDistributionConstant;

/**
 * @author Patrick Heusser
 */
public class DeployAndReleaseAction extends AbstractBusinessAction {

	private RandomDistribution deploymentSuccess = new RandomDistributionConstant(0, 100);;

	/**
	 * @see com.x8ing.mc.bp.AbstractBusinessAction#execute(com.x8ing.lsm4j.state.ProcessableState,
	 *      com.x8ing.mc.bp.BusinessContext, com.x8ing.lsm4j.Condition, java.util.List)
	 */
	public void execute(ProcessableState currentState, BusinessContext businessContext, Condition previousCondition,
			List lastVisitedStatesHistory) {

		boolean deployedWithSucess = (deploymentSuccess.getNextRandomNumber() < businessContext.getConfiguration()
				.getChanceDeployAndRealeaseDeploymentSuccess() ? true : false);

		// set it on context (for condition)
		businessContext.setDeployedWithSucess(deployedWithSucess);

		if (deployedWithSucess) {
			addBalanceSheetTransaction("deployment of new software (with success)", -businessContext.getConfiguration()
					.getCostActionDeployAndRelease(), BalanceAccount.DEPLOYMENT);

			// change bug state
			StringBuffer sb = new StringBuffer();
			sb.append("deployed now bugs with IDs:");

			List bugsToDeploy = businessContext.getBugs().getBugsWithState(Bug.BugState.STATE_BUG_TEST_SUCESS);

			double totalProcessImprove = 0;

			for (Iterator it = bugsToDeploy.iterator(); it.hasNext();) {
				Bug bug = (Bug) it.next();
				bug.setBugState(Bug.BugState.STATE_BUG_DEPLOYED);
				sb.append(bug.getBugID());
				sb.append(" ");

				double loss = bug.getNegativImpactOnBusinessProcess();
				if (loss < 0) {
					// remove this if sure that working
					throw new RuntimeException("negativ loss");
				}

				totalProcessImprove += loss;
			}

			double processGainOld = businessContext.getCurrentProcessGain();

			// since we released the bugs, the process works now more efficient
			// therefore: update process value
			businessContext.setProcessLossCausedByBugsCurrentValue(businessContext.getProcessLossCausedByBugsCurrentValue()
					- totalProcessImprove);

			double processGainNew = businessContext.getCurrentProcessGain();

			addLogBookEntry("the new release improved the business process. gainOld:" + Math.round(processGainOld) + " gainNew:"
					+ Math.round(processGainNew));

		} else {
			addBalanceSheetTransaction("deployment of new software (with failure)", -businessContext.getConfiguration()
					.getCostActionDeployAndRelease(), BalanceAccount.DEPLOYMENT);

		}

	}

	protected void lazyInit() {
		// empty
	}

}
