/*
 * Created on Aug 29, 2007
 */
package com.x8ing.mc.bp.develop;

import java.util.Iterator;
import java.util.List;

import com.x8ing.lsm4j.Condition;
import com.x8ing.lsm4j.state.ProcessableState;
import com.x8ing.mc.bp.AbstractBusinessAction;
import com.x8ing.mc.bp.BalanceAccount;
import com.x8ing.mc.bp.Bug;
import com.x8ing.mc.bp.BusinessContext;
import com.x8ing.mc.distribution.RandomDistribution;
import com.x8ing.mc.distribution.RandomDistributionConstant;

/**
 * @author Patrick Heusser
 */
public class FixAllKnownBugsAction extends AbstractBusinessAction {

	private RandomDistribution fixingSucess = new RandomDistributionConstant(0, 100);

	/**
	 * all bugs in that state must be fixed.
	 */
	public static final Bug.BugState[] STATES_TO_FIX = new Bug.BugState[] { Bug.BugState.STATE_BUG_NEW,
			Bug.BugState.STATE_BUG_FIX_LATER, Bug.BugState.STATE_BUG_TEST_FAILED };

	/**
	 * @see com.x8ing.mc.bp.AbstractBusinessAction#execute(com.x8ing.lsm4j.state.ProcessableState,
	 *      com.x8ing.mc.bp.BusinessContext, com.x8ing.lsm4j.Condition, java.util.List)
	 */
	public void execute(ProcessableState currentState, BusinessContext businessContext, Condition previousCondition,
			List lastVisitedStatesHistory) {

		// prepare bug list
		List allBugs = businessContext.getBugs().getBugsWithState(STATES_TO_FIX);

		if (allBugs.size() <= 0) {
			// inconsistent model! we never get here if we have nothing to do.
			throw new RuntimeException("inconsistent model state.");
		}

		addLogBookEntry("Go to fix all yet known bugs (" + allBugs.size()
				+ ") (bugs with states: new, fixLater, productionTestFailed.)");

		for (Iterator it = allBugs.iterator(); it.hasNext();) {

			Bug bug = (Bug) it.next();
			double fixingCost = -bug.getEstimatedCostToFixBug();

			// test if fixing did work out
			boolean bugFixingWorkedOut = (fixingSucess.getNextRandomNumber() < businessContext.getConfiguration()
					.getChanceFixBugWithSuccess() ? true : false);
			if (bugFixingWorkedOut) {

				bug.setBugState(Bug.BugState.STATE_BUG_FIXED);
				addLogBookEntry("Fixed bug (id=" + bug.getBugID() + "). State is now: " + bug.getBugState());
				addBalanceSheetTransaction("costs for fix of bug (id=" + bug.getBugID() + ")", fixingCost, BalanceAccount.DEVELOP);

			} else {

				addLogBookEntry("Fixing of bug failed (id=" + bug.getBugID() + ", state=" + bug.getBugState() + ").");
				addBalanceSheetTransaction("costs for failed bug-fix (id=" + bug.getBugID() + ")", fixingCost,
						BalanceAccount.DEVELOP);

			}

		}

	}

	protected void lazyInit() {
		// empty
	}

}
