/*
 * Created on Aug 29, 2007
 */
package com.x8ing.mc.bp.operation;

import java.util.List;

import com.x8ing.lsm4j.Condition;
import com.x8ing.lsm4j.state.ProcessableState;
import com.x8ing.mc.bp.AbstractBusinessAction;
import com.x8ing.mc.bp.BalanceAccount;
import com.x8ing.mc.bp.Bug;
import com.x8ing.mc.bp.BusinessContext;
import com.x8ing.mc.distribution.RandomDistribution;
import com.x8ing.mc.distribution.RandomDistributionConstant;

/**
 * @author Patrick Heusser
 */
public class OperationRunsAction extends AbstractBusinessAction {

	private RandomDistribution outageChance = new RandomDistributionConstant(0, 100);

	private RandomDistribution findNewBugChance = new RandomDistributionConstant(0, 100);

	/**
	 * @see com.x8ing.mc.bp.AbstractBusinessAction#execute(com.x8ing.lsm4j.state.ProcessableState,
	 *      com.x8ing.mc.bp.BusinessContext, com.x8ing.lsm4j.Condition, java.util.List)
	 */
	public void execute(ProcessableState currentState, BusinessContext businessContext, Condition previousCondition,
			List lastVisitedStatesHistory) {

		// check if we found a new bug
		if (findNewBugChance.getNextRandomNumber() > businessContext.getConfiguration().getChanceForFindingANewBugInProduction()
				&& !businessContext.isNoMoreDiscoveringOfBugs()) {

			// add a new bug to the queue (create the bug with value 0 first)
			Bug bug = new Bug(businessContext.getCurrentDate(), 0, 0, businessContext.getBugs().getNextBugID());
			businessContext.getQueuedBugs().add(bug);
			addLogBookEntry("production discovered a new bug. bugID=" + bug.getBugID());

		}

		// check if an outtage happens
		if (outageChance.getNextRandomNumber() < businessContext.getConfiguration().getChanceOperationHasOutage()) {

			// we have a problem
			businessContext.setProductionSystemRunning(false);

			addLogBookEntry("production failure occured.");

		} else {

			// business runs smoothly
			addBalanceSheetTransaction("current process gain", businessContext.getCurrentProcessGain(), BalanceAccount.PRODUCTION);

		}

	}

	protected void lazyInit() {
		// empty
	}

}
