/*
 * Created on Aug 30, 2007
 */
package com.x8ing.mc.distribution;

import cern.jet.random.Normal;
import cern.jet.random.engine.MersenneTwister;

/**
 * Uses an open library by cern.ch
 * <p>
 * http://spi.cern.ch/extsoft/packages.php?pkg=Colt
 * <p>
 * http://dsd.lbl.gov/~hoschek/colt/
 * 
 * @author Patrick Heusser
 */
public class RandomDistributionGauss implements RandomDistribution {

	private Normal normal = null;

	private static final int MAX_SEARCH_ITERATIONS = 5000;

	private double standardDeviation = 0;

	private double min = -1;

	private double max = -1;

	/**
	 * <p>
	 * 
	 * nStdDeviations = percentage <br>
	 * 1 = 0.682689492137 <br>
	 * 2 = 0.954499736104 <br>
	 * 3 = 0.997300203937 <br>
	 * 4 = 0.999936657516 <br>
	 * 5 = 0.999999426697 <br>
	 * 6 = 0.999999998027
	 * 
	 * @param min
	 *            the minimal return value
	 * @param max
	 *            the maximum return value
	 * @param nStdDeviations:
	 *            e.g. 1,2,3 etc... as described above..
	 * @return a value within range, even if it's hard bound!
	 */
	public RandomDistributionGauss(double min, double max, int nStdDeviations) {

		super();

		normal = new Normal(-1, 1, new MersenneTwister());

		init(min, max, nStdDeviations);

	}

	public void init(double min, double max, int nStdDeviations) {

		if (nStdDeviations < 0.5 || nStdDeviations > 20) {

			throw new IllegalStateException("error while initialization. 'stdDeviation must be within 0.5-6"
					+ ". value where: min=" + min + " max=" + max + " nStdDeviations=" + nStdDeviations);

		}

		double mean = (((double) (max + min)) / 2);

		this.standardDeviation = (max - mean) / nStdDeviations;
		this.max = max;
		this.min = min;

		if (min >= max) {
			throw new IllegalStateException("error while initialization. 'min' value must be smaller "
					+ "then 'max' value. but values where: min=" + min + " max=" + max + " nStdDeviations=" + nStdDeviations);
		}

		normal.setState(mean, standardDeviation);

	}

	/**
	 * @see com.x8ing.mc.distribution.RandomDistribution#getNextRandomNumber()
	 */
	public double getNextRandomNumber() {

		double ret = 0;

		int searchloop = 0;

		do {
			searchloop++;
			ret = normal.nextDouble();

			// limit the endless loop
			if (searchloop >= MAX_SEARCH_ITERATIONS) {
				ret = min;
				System.err.println("warning, searched to long for gaussian number in hard limited range."
						+ " please adjust numbers. min=" + min + " max=" + max + " standardDeviation=" + standardDeviation);
				break;
			}

		} while ((ret > max || ret < min));

		return ret;

	}

}
