/*
 * Created on Sept 21, 2007
 *
 */
package com.x8ing.mc.web;

import java.io.IOException;

import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.statistics.HistogramDataset;

import com.x8ing.mc.MonteCarloControler;

/**
 * Servlet for executing the MonteCarlo simulation process.
 * 
 * @author Patrick Heusser
 */
public class HistogramImageServlet extends HttpServlet {

	private static final long serialVersionUID = 1L;

	public HistogramImageServlet() {
		super();
	}

	protected void doGet(HttpServletRequest arg0, HttpServletResponse arg1) throws ServletException, IOException {
		doPost(arg0, arg1);
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		setNonCachingHeaders(response);

		MonteCarloControler mcc = (MonteCarloControler) request.getSession().getAttribute(
				ProcessMCAction.SESSION_KEY_MONTE_CARLO_CONTRLOER);

		HistogramDataset data = new HistogramDataset();
		data.addSeries("occurence", mcc.getBalanceValues(), 100);

		JFreeChart chart = ChartFactory.createHistogram("Earnings Histogram", "earning", "number of occurrences", data,
				PlotOrientation.VERTICAL, false, false, false);

		try {

			ChartUtilities.writeChartAsPNG(response.getOutputStream(), chart, 800, 600);

		} catch (IOException e) {
			System.err.println("Problem occurred creating chart.");
			e.printStackTrace();
			response.getOutputStream().print(e.toString());

		}

	}

	private void setNonCachingHeaders(HttpServletResponse response) {

		response.setHeader("Expires", "Sat, 6 May 1995 12:00:00 GMT");

		// Set standard HTTP/1.1 no-cache headers.
		response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");

		// Set IE extended HTTP/1.1 no-cache headers (use addHeader).
		response.addHeader("Cache-Control", "post-check=0, pre-check=0");

		// Set standard HTTP/1.0 no-cache header.
		response.setHeader("Pragma", "no-cache");
	}

}