/*
 * Created on Aug 29, 2007
 *
 */
package com.x8ing.mc.bp;

import java.util.GregorianCalendar;

import junit.framework.TestCase;

/**
 * @author Patrick Heusser
 */
public class BalanceSheetTest extends TestCase {

    public void test1() {

        BalanceSheet balanceSheet = new BalanceSheet();

        assertTrue(balanceSheet.calculateCurrentBalance() == 0);

        double t1 = +1.2;
        double t2 = +10;
        double t3 = +100;
        double t4 = -1000;
        double t5 = +10000;
        double t6 = +44;
        double t7 = 444;

        // make sure we start with 0
        assertTrue(0 == balanceSheet.calculateCurrentBalance());

        // add transactions
        balanceSheet.addTransaction("booking1", t1, GregorianCalendar.getInstance(),
                BalanceAccount.BUG_ANALYZING);
        balanceSheet.addTransaction("booking2", t2, GregorianCalendar.getInstance(),
                BalanceAccount.DEPLOYMENT);
        balanceSheet.addTransaction("booking3", t3, GregorianCalendar.getInstance(), BalanceAccount.DEVELOP);

        // check grand total
        assertTrue(t1 + t2 + t3 == balanceSheet.calculateCurrentBalance());

        // check accounts total
        assertTrue(t1 == balanceSheet
                .calculateCurrentBalanceForAccounts(new BalanceAccount[] { BalanceAccount.BUG_ANALYZING }));
        assertTrue(t1 + t3 == balanceSheet.calculateCurrentBalanceForAccounts(new BalanceAccount[] {
                BalanceAccount.BUG_ANALYZING, BalanceAccount.DEVELOP }));

        // more transactions
        balanceSheet.addTransaction("booking4", t4, GregorianCalendar.getInstance(),
                BalanceAccount.BUG_ANALYZING);
        balanceSheet.addTransaction("booking5", t5, GregorianCalendar.getInstance(),
                BalanceAccount.BUG_ANALYZING);

        // check grand total
        assertTrue(t1 + t2 + t3 + t4 + t5 == balanceSheet.calculateCurrentBalance());

        // check account total
        assertTrue(t1 + t4 + t5 == balanceSheet
                .calculateCurrentBalanceForAccounts(new BalanceAccount[] { BalanceAccount.BUG_ANALYZING }));
        assertTrue(t3 == balanceSheet
                .calculateCurrentBalanceForAccounts(new BalanceAccount[] { BalanceAccount.DEVELOP }));

        // check for non existing zero value
        assertTrue(0 == balanceSheet
                .calculateCurrentBalanceForAccounts(new BalanceAccount[] { BalanceAccount.TESTING }));
        assertTrue(0 == balanceSheet
                .calculateCurrentBalanceForAccounts(new BalanceAccount[] { BalanceAccount.PRODUCTION }));
        assertTrue(0 == balanceSheet.calculateCurrentBalanceForAccounts(new BalanceAccount[] {
                BalanceAccount.TESTING, BalanceAccount.PRODUCTION }));

        // more transactions
        balanceSheet.addTransaction("booking6", t6, GregorianCalendar.getInstance(),
                BalanceAccount.DEPLOYMENT);
        balanceSheet.addTransaction("booking7", t7, GregorianCalendar.getInstance(),
                BalanceAccount.DEPLOYMENT);

        // check grand total
        assertTrue(t1 + t2 + t3 + t4 + t5 + t6 + t7 == balanceSheet.calculateCurrentBalance());

        // check account total
        assertTrue(0 == balanceSheet
                .calculateCurrentBalanceForAccounts(new BalanceAccount[] { BalanceAccount.TESTING }));
        assertTrue(0 == balanceSheet
                .calculateCurrentBalanceForAccounts(new BalanceAccount[] { BalanceAccount.PRODUCTION }));
        assertTrue(0 == balanceSheet.calculateCurrentBalanceForAccounts(new BalanceAccount[] {
                BalanceAccount.TESTING, BalanceAccount.PRODUCTION }));

        // check accounts
        assertTrue(t2 + t6 + t7 == balanceSheet
                .calculateCurrentBalanceForAccounts(new BalanceAccount[] { BalanceAccount.DEPLOYMENT }));
        assertTrue(t2 + t6 + t7 + t1 + t4 + t5 == balanceSheet
                .calculateCurrentBalanceForAccounts(new BalanceAccount[] { BalanceAccount.DEPLOYMENT,
                        BalanceAccount.BUG_ANALYZING }));

        // test if print generally works without error
        String print1 = balanceSheet.printBalanceSheetByAccount(false);
        assertNotNull(print1);
        assertTrue(print1.length() > 0);
        System.out.println(print1);

        String print2 = balanceSheet.printBalanceSheetByDate(false);
        assertNotNull(print2);
        assertTrue(print2.length() > 0);
        System.out.println(print2);
        
        String print3 = balanceSheet.printBalanceSheetAccountOverview();
        assertNotNull(print3);
        assertTrue(print3.length() > 0);
        System.out.println(print3);
        

    }
}
