/*
 * Created on May 14, 2007
 */
package com.x8ing.lsm4j.processable;

import java.util.List;

import junit.framework.TestCase;

import com.x8ing.lsm4j.Condition;
import com.x8ing.lsm4j.Action;
import com.x8ing.lsm4j.StateContext;
import com.x8ing.lsm4j.state.AbstractProcessableGraphException;
import com.x8ing.lsm4j.state.MaximumIterationsReachedException;
import com.x8ing.lsm4j.state.NoMatchingTransitionConditionFoundException;
import com.x8ing.lsm4j.state.ProcessableGraph;
import com.x8ing.lsm4j.state.ProcessableState;
import com.x8ing.lsm4j.std.TracingGraphListener;

/**
 *
 * @author Patrick Heusser
 */
public class ProcessableGraphTest1 extends TestCase {

	/**
	 * Test on graph1 with same action.
	 */
	public void test1() {

		// create graph instance
		ProcessableGraph graph = new ProcessableGraph();
		graph.registerGraphListener(new TracingGraphListener(System.out));

		IncrementAction incrementAction = new IncrementAction();

		final String STATE_E = "stateE"; // to check for equality
		final String STATE_F = "stateF"; // to check for equality

		// create states
		ProcessableState stateA = new ProcessableState(1, "stateA", incrementAction, false); // start
		ProcessableState stateB = new ProcessableState(2, "stateB", incrementAction, false);
		ProcessableState stateC = new ProcessableState(3, "stateC", incrementAction, false);
		ProcessableState stateD = new ProcessableState(4, "stateD", incrementAction, false);
		ProcessableState stateE = new ProcessableState(5, STATE_E, incrementAction, true);
		ProcessableState stateF = new ProcessableState(6, STATE_F, incrementAction, true);

		// attach transitions: going from A
		graph.addValidTransition(stateA, stateB, new SimpleNumberCondition(0));
		graph.addValidTransition(stateA, stateC, new SimpleNumberCondition(2));
		graph.addValidTransition(stateA, stateD, new SimpleNumberCondition(5));

		// attach transitions: going from B
		graph.addValidTransition(stateB, stateB, new SimpleNumberCondition(4)); // circle
		graph.addValidTransition(stateB, stateE, new SimpleNumberCondition(5));
		graph.addValidTransition(stateB, stateC, new SimpleNumberCondition(1));
		graph.addValidTransition(stateB, stateC, new SimpleNumberCondition(20));

		// attach transitions: going from C
		graph.addValidTransition(stateC, stateD, new SimpleNumberCondition(2));
		graph.addValidTransition(stateC, stateF, new SimpleNumberCondition(4));
		graph.addValidTransition(stateC, stateB, new SimpleNumberCondition(3));
		graph.addValidTransition(stateC, stateF, new SimpleNumberCondition(21));

		// attach transitions: going from D
		graph.addValidTransition(stateD, stateD, new SimpleNumberCondition(6)); // circle
		graph.addValidTransition(stateD, stateD, new SimpleNumberCondition(7)); // circle
		graph.addValidTransition(stateD, stateC, new SimpleNumberCondition(3));

		System.out.println(graph + " \n\n");

		{
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = -1;
			try {

				ProcessableState lastState = graph.run(1, context);

				assertEquals(5, context.currentNumber);

				assertEquals(lastState.getUniqueID(), 6);
				assertEquals(lastState.getDescription(), STATE_F);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

		{
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 1;
			try {

				consoleNextTest();
				ProcessableState lastState = graph.run(1, context);

				assertEquals(6, context.currentNumber);

				assertEquals(lastState.getUniqueID(), 5);
				assertEquals(lastState.getDescription(), STATE_E);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

		{
			boolean ex = false;
			// starting with 5 leads into no more way possible in stateD
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 4;
			try {

				consoleNextTest();
				graph.run(1, context);

				// should not be reached.
				fail();

			} catch (NoMatchingTransitionConditionFoundException e) {
				// ok!
				ex = true;

			} catch (MaximumIterationsReachedException e) {
				fail();
			} catch (Exception e) {
				fail();
			}
			assertTrue(ex);
		}

		{
			boolean ex = false;
			// starting with 500 leads into no more way possible in stateD
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 500;
			try {

				consoleNextTest();
				graph.run(1, context);

				// should not be reached.
				fail();

			} catch (NoMatchingTransitionConditionFoundException e) {
				// ok!
				ex = true;
			} catch (MaximumIterationsReachedException e) {
				fail();
			} catch (Exception e) {
				fail();
			}
			assertTrue(ex);
		}

		{
			// starting at a different node
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 19;
			try {

				consoleNextTest();
				graph.run(2, context);

				assertEquals(22, context.currentNumber);

			} catch (AbstractProcessableGraphException e) {
				fail();
			} catch (Exception e) {
				fail();
			}
		}

	}

	/**
	 * Test on graph2 with different actions.
	 */

	public void test2() {

		ProcessableGraph graph = createGraph2();

		System.out.println(graph + " \n\n");

		{
			// starting with 0 leads into 37
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 0;
			try {

				graph.run(1, context);
				assertEquals(37, context.currentNumber);

			} catch (AbstractProcessableGraphException e) {
				fail();
			}
		}

		{
			consoleNextTest();

			// starting with 0 leads into 37
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 1;
			try {

				graph.run(1, context);
				assertEquals(27, context.currentNumber);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

	}

	public void test2StepLimit() {

		ProcessableGraph graph = createGraph2();

		System.out.println(graph + " \n\n");

		{

			ProcessableState state = null;

			NumberStateContext context = new NumberStateContext();
			try {

				// test by going only limited steps on the graph
				consoleNextTest();
				context.currentNumber = 1;
				state = graph.run(1, context, 1);
				assertEquals(3, state.getUniqueID());

				consoleNextTest();
				context.currentNumber = 1;
				state = graph.run(1, context, 2);
				assertEquals(2, state.getUniqueID());

				consoleNextTest();
				context.currentNumber = 1;
				state = graph.run(1, context, 3);
				assertEquals(4, state.getUniqueID());

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}
	}

	/**
	 * Test the processing of the graph in the stop and go mode, this means, we start it once, interrupt it, and
	 * continue later. We expect that the state and position of the graph will be retored.
	 *
	 */
	public void test2StopAndGoProcessing() {

		ProcessableGraph graph = createGraph2();

		System.out.println(graph + " \n\n");

		{

			ProcessableState state = null;

			NumberStateContext context = new NumberStateContext();
			consoleNextTest();
			context.currentNumber = 1;

			try {

				// initial go for one transtion
				state = graph.run(1, context, 1);
				assertEquals(3, state.getUniqueID());

				// now just continue with one step
				state = graph.runContinue(1);
				assertEquals(2, state.getUniqueID());
				assertEquals(10, context.currentNumber);

				// now just continue with one step
				state = graph.runContinue(1);
				assertEquals(4, state.getUniqueID());
				assertEquals(15, context.currentNumber);

				// now just continue with one step
				try {
					state = graph.runContinue(1);
					assertNull(state);
				} catch (Exception e) {
					fail();
				}

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

	}

	private ProcessableGraph createGraph2() {
		ProcessableGraph graph = new ProcessableGraph();
		graph.registerGraphListener(new TracingGraphListener(System.out));

		IncrementAction incrementAction = new IncrementAction();
		incrementAction.delta = 2;

		IncrementAction action2 = new IncrementAction();
		action2.delta = 5;

		IncrementAction action3 = new IncrementAction();
		action3.delta = 7;

		IncrementAction action4 = new IncrementAction();
		action4.delta = 12;

		IncrementAction action5 = new IncrementAction();
		action5.delta = 23;

		ProcessableState stateA = new ProcessableState(1, "stateA", incrementAction, false);
		ProcessableState stateB = new ProcessableState(2, "stateB", action2, false);
		ProcessableState stateC = new ProcessableState(3, "stateC", action3, false);
		ProcessableState stateD = new ProcessableState(4, "stateD", action4, true);
		ProcessableState stateE = new ProcessableState(5, "stateE", action5, true);

		// going from A
		graph.addValidTransition(stateA, stateB, new SimpleNumberCondition(2));
		graph.addValidTransition(stateA, stateC, new SimpleNumberCondition(3));

		// going from B
		graph.addValidTransition(stateB, stateD, new SimpleNumberCondition(15));
		graph.addValidTransition(stateB, stateC, new SimpleNumberCondition(7));

		// going from C
		graph.addValidTransition(stateC, stateB, new SimpleNumberCondition(10));
		graph.addValidTransition(stateC, stateE, new SimpleNumberCondition(14));
		return graph;
	}

	/**
	 * Infinite loop must be interrupted.
	 */
	public void testInfiniteLoop() {

		ProcessableGraph graph = new ProcessableGraph();
		graph.registerGraphListener(new TracingGraphListener(System.out));

		IncrementAction incrementAction = new IncrementAction();
		incrementAction.delta = 1;

		IncrementAction action2 = new IncrementAction();
		action2.delta = -1;

		ProcessableState stateA = new ProcessableState(1, "stateA", incrementAction, false);
		ProcessableState stateB = new ProcessableState(2, "stateB", action2, false);
		ProcessableState stateC = new ProcessableState(3, "stateC", action2, true);

		// going from A
		graph.addValidTransition(stateA, stateB, new SimpleNumberCondition(1));
		graph.addValidTransition(stateB, stateA, new SimpleNumberCondition(0));
		graph.addValidTransition(stateB, stateC, new SimpleNumberCondition(2));

		// important: add the loop limitation
		graph.setMaximumLoops(10);

		System.out.println(graph + " \n\n");

		{

			boolean ex = false;
			consoleNextTest();

			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 0;
			try {

				graph.run(1, context);
				fail();

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				ex = true;
			}

			assertTrue(ex);
		}

	}

	private void consoleNextTest() {
		System.out.println("\n\n-------------------------");
	}

	private static void trace(String msg) {
		System.out.println("\t " + msg);

	}

	private static class IncrementAction implements Action {

		private int delta = 1; // how much to increment

		private ProcessableState currentState = null; // for testcase

		public void execute(ProcessableState currentState, StateContext stateContext, Condition previousCriteria,
				List lastVisitedStatesHistory) {

			this.currentState = currentState;

			// do action logic: cast and increment
			NumberStateContext numberStateContext = (NumberStateContext) stateContext;
			numberStateContext.currentNumber += delta;
			trace("Action process: changed currentNumber to " + numberStateContext.currentNumber);

		}

	}

	private static class NumberStateContext implements StateContext {

		int currentNumber = -1;

	}

	private static class SimpleNumberCondition implements Condition {

		private int verbalConditionCriteria = 0;

		public SimpleNumberCondition(int simpleCriteria) {
			this.verbalConditionCriteria = simpleCriteria;
		}

		public String traceInfo() {
			return "verbalConditionCriteria=" + verbalConditionCriteria;
		}

		public boolean conditionTrue(StateContext currentContext) {
			return verbalConditionCriteria == ((NumberStateContext) currentContext).currentNumber;
		}

	}

}
