/*
 * Created on May 14, 2007
 */
package com.x8ing.lsm4j.processable;

import java.util.List;

import junit.framework.TestCase;

import com.x8ing.lsm4j.Action;
import com.x8ing.lsm4j.Condition;
import com.x8ing.lsm4j.StateContext;
import com.x8ing.lsm4j.state.MaximumIterationsReachedException;
import com.x8ing.lsm4j.state.NoMatchingTransitionConditionFoundException;
import com.x8ing.lsm4j.state.ProcessableGraph;
import com.x8ing.lsm4j.state.ProcessableState;
import com.x8ing.lsm4j.std.EmptyAction;
import com.x8ing.lsm4j.std.TracingGraphListener;
import com.x8ing.lsm4j.std.TrueCondition;

/**
 *
 * @author Patrick Heusser
 */
public class ProcessableGraphTest3 extends TestCase {

	/**
	 * Test on graph1 with same action.
	 */
	public void test1() {

		ProcessableGraph graph = new ProcessableGraph();
		graph.registerGraphListener(new TracingGraphListener(System.out));

		final String STATE_D = "stateD"; // to check for equality

		// build graph layout
		ProcessableState stateA = new ProcessableState(1, "stateA", new IncrementAction(1), false); // start
		ProcessableState stateB = new ProcessableState(2, "stateB", new IncrementAction(3), false);
		ProcessableState stateC = new ProcessableState(3, "stateC", new IncrementAction(5), false);
		ProcessableState stateD = new ProcessableState(4, STATE_D, new EmptyAction(), true); // end

		// going from A
		graph.addValidTransition(stateA, stateB, new NumberCondition(1), new IncrementAction(10));
		graph.addValidTransition(stateA, stateB, new NumberCondition(2));
		graph.addValidTransition(stateA, stateB, new NumberCondition(3), new IncrementAction(20));
		graph.addValidTransition(stateA, stateC, new NumberCondition(4));
		graph.addValidTransition(stateA, stateC, new NumberCondition(5), new IncrementAction(30));

		// going from B
		graph.addValidTransition(stateB, stateD, new TrueCondition());

		// going from C
		graph.addValidTransition(stateC, stateB, new NumberCondition(9));
		graph.addValidTransition(stateC, stateD, new NumberCondition(40));

		System.out.println(graph + " \n\n");

		{
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 0;
			try {

				ProcessableState lastState = graph.run(1, context);

				assertEquals(14, context.currentNumber);

				assertEquals(lastState.getUniqueID(), 4);
				assertEquals(lastState.getDescription(), STATE_D);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

		{
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 1;
			try {

				ProcessableState lastState = graph.run(1, context);

				assertEquals(5, context.currentNumber);

				assertEquals(lastState.getUniqueID(), 4);
				assertEquals(lastState.getDescription(), STATE_D);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

		{
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 2;
			try {

				ProcessableState lastState = graph.run(1, context);

				assertEquals(26, context.currentNumber);

				assertEquals(lastState.getUniqueID(), 4);
				assertEquals(lastState.getDescription(), STATE_D);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

		{
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 3;
			try {

				ProcessableState lastState = graph.run(1, context);

				assertEquals(12, context.currentNumber);

				assertEquals(lastState.getUniqueID(), 4);
				assertEquals(lastState.getDescription(), STATE_D);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

		{
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 4;
			try {

				ProcessableState lastState = graph.run(1, context);

				assertEquals(40, context.currentNumber);

				assertEquals(lastState.getUniqueID(), 4);
				assertEquals(lastState.getDescription(), STATE_D);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

		{
			NumberStateContext context = new NumberStateContext();
			context.currentNumber = 6;
			boolean flow = false;
			try {

				ProcessableState lastState = graph.run(1, context);

				// should not reach here, since there is no valid condition
				flow = true;
				fail();

			} catch (NoMatchingTransitionConditionFoundException e) {

				// it's ok to catch this exception

			} catch (MaximumIterationsReachedException e) {
				fail();
			}
			assertFalse(flow);
		}

	}

	private void consoleNextTest() {
		System.out.println("\n\n-------------------------");
	}

	private static void trace(String msg) {
		System.out.println("\t " + msg);

	}

	private static class IncrementAction implements Action {

		int delta = 1;

		public IncrementAction(int delta) {
			this.delta = delta;
		}

		public void execute(ProcessableState currentState, StateContext stateContext, Condition previousCriteria,
				List lastVisitedStatesHistory) {

			if (stateContext instanceof NumberStateContext) {
				NumberStateContext numberStateContext = (NumberStateContext) stateContext;
				numberStateContext.currentNumber += delta;
				trace("Action process: changed currentNumber to " + numberStateContext.currentNumber);

			}

		}

	}

	private static class NumberStateContext implements StateContext {

		int currentNumber = -1;

	}

	private static class NumberCondition implements Condition {

		private int simpleCriteria = 0;

		/**
		 * @param simpleCriteria
		 */
		public NumberCondition(int simpleCriteria) {
			super();
			this.simpleCriteria = simpleCriteria;
		}

		public String traceInfo() {
			return "simpleCriteria=" + simpleCriteria;
		}

		public int criteriaHashCode() {
			return simpleCriteria;
		}

		public String toString() {
			return String.valueOf(simpleCriteria);
		}

		public boolean conditionTrue(StateContext currentContext) {
			return simpleCriteria == ((NumberStateContext) currentContext).currentNumber;
		}

	}

}
