/*
 * Created on May 14, 2007
 */
package com.x8ing.lsm4j.processable;

import java.util.List;
import java.util.Random;

import com.x8ing.lsm4j.Condition;
import com.x8ing.lsm4j.Action;
import com.x8ing.lsm4j.StateContext;
import com.x8ing.lsm4j.StateMetaInformation;
import com.x8ing.lsm4j.state.MaximumIterationsReachedException;
import com.x8ing.lsm4j.state.NoMatchingTransitionConditionFoundException;
import com.x8ing.lsm4j.state.ProcessableGraph;
import com.x8ing.lsm4j.state.ProcessableState;
import com.x8ing.lsm4j.std.EmptyAction;
import com.x8ing.lsm4j.std.TracingGraphListener;
import com.x8ing.lsm4j.std.TrueCondition;

import junit.framework.TestCase;

/**
 *
 * @author Patrick Heusser
 */
public class ProcessableGraphTest3MorseCode extends TestCase {

	private static boolean TRACE_TO_CONSOLE = true;

	/**
	 * Tests one character decoding only
	 */
	public void test1() {

		ProcessableGraph graph = buildMorseStateMachine();

		trace(graph.toString());

		// Test character T
		{
			MorseContext tempContext = new MorseContext();
			// TODO space at end
			tempContext.morseTextCoded = "- ";

			try {

				graph.run(1, tempContext);

				String decoded = tempContext.morseTextDecoded.toString().trim();

				assertEquals("T", decoded);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

	}

	/**
	 * Tests an illegal character
	 */
	public void testIllegalCharacter() {

		ProcessableGraph graph = buildMorseStateMachine();

		trace(graph.toString());

		// Test character T
		{
			MorseContext tempContext = new MorseContext();
			// TODO space at end
			tempContext.morseTextCoded = "---...";

			boolean check = true;
			try {

				graph.run(1, tempContext);

				tempContext.morseTextDecoded.toString().trim();

				check = false;
				fail();

			} catch (NoMatchingTransitionConditionFoundException e) {

				// OK , since syntax is illegal!

			} catch (MaximumIterationsReachedException e) {
				fail();
			}

			assertTrue(check);
		}

	}

	/**
	 * Test all characters of alphabet at once.
	 *
	 */
	public void testAllCharacters() {

		// quickly add all morse codes for testing with graph
		String[][] morseCodes = morseCodes();

		ProcessableGraph morseStateMachine = buildMorseStateMachine();

		for (int i = 0; i < morseCodes.length; i++) {

			String[] enc = morseCodes[i];
			String character = enc[0];
			String morse = enc[1];

			MorseContext tempContext = new MorseContext();
			// TODO space at end !
			tempContext.morseTextCoded = morse + " ";

			try {

				morseStateMachine.run(1, tempContext);

				String decoded = tempContext.morseTextDecoded.toString().trim();

				assertEquals(character, decoded);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}

		}

	}

	public void testSequenceofCharacters() {

		String morseSequence = "... --- ... "; // SOS

		ProcessableGraph morseStateMachine = buildMorseStateMachine();
		morseStateMachine.registerGraphListener(new TracingGraphListener(System.out));

		// Test character T
		{
			MorseContext tempContext = new MorseContext();
			// TODO space at end
			tempContext.morseTextCoded = morseSequence;

			try {

				morseStateMachine.run(1, tempContext);

				String decoded = tempContext.morseTextDecoded.toString().trim();

				assertEquals("SOS", decoded);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

	}

	public void testTwoWords() {

		String morseSequence = "... --- ...   -.-- --- ..- "; // SOS YOU

		ProcessableGraph morseStateMachine = buildMorseStateMachine();
		morseStateMachine.registerGraphListener(new TracingGraphListener(System.out));

		// Test character T
		{
			MorseContext tempContext = new MorseContext();
			// TODO space at end
			tempContext.morseTextCoded = morseSequence;

			try {

				morseStateMachine.run(1, tempContext);

				String decoded = tempContext.morseTextDecoded.toString().trim();

				assertEquals("SOS YOU", decoded);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

	}

	/**
	 * test the graph with a long paragraph.
	 */
	public void testLongParagraph() {

		String text = "Lorem ipsum dolor sit amet consectetuer adipiscing elit Vestibulum et "
				+ "pede et risus condimentum hendrerit Lorem ipsum dolor sit amet consectetuer "
				+ "adipiscing elit Integer molestie lorem a mi Class aptent taciti sociosqu ad "
				+ "litora torquent per conubia nostra per inceptos hymenaeos Aliquam non enim Donec "
				+ "feugiat tristique nibh Integer magna justo elementum eu lacinia vel iaculis quis "
				+ "elit Morbi at sapien Morbi eget ipsum congue nisl blandit mollis Sed justo libero "
				+ "commodo in vehicula a pretium facilisis neque Maecenas iaculis lacus id elit Nulla "
				+ "facilisi Aliquam erat volutpat Vestibulum libero ante venenatis quis viverra ac sodales "
				+ "eget nisl Nulla ornare vulputate nulla Nullam quis mi Sed eu dui";

		String morseSequence = ".-.. --- .-. . --   .. .--. ... ..- --   -.. --- .-.. --- .-.   ... .. - "
				+ "  .- -- . -   -.-. --- -. ... . -.-. - . - ..- . .-.   .- -.. .. .--. .. ... -.-. .. -. --. "
				+ "  . .-.. .. -   ...- . ... - .. -... ..- .-.. ..- --   . -   .--. . -.. .   . - "
				+ "  .-. .. ... ..- ...   -.-. --- -. -.. .. -- . -. - ..- --   .... . -. -.. .-. . .-. .. - "
				+ "  .-.. --- .-. . --   .. .--. ... ..- --   -.. --- .-.. --- .-.   ... .. -   .- -- . - "
				+ "  -.-. --- -. ... . -.-. - . - ..- . .-.   .- -.. .. .--. .. ... -.-. .. -. --. "
				+ "  . .-.. .. -   .. -. - . --. . .-.   -- --- .-.. . ... - .. .   .-.. --- .-. . -- "
				+ "  .-   -- ..   -.-. .-.. .- ... ...   .- .--. - . -. -   - .- -.-. .. - ..  "
				+ " ... --- -.-. .. --- ... --.- ..-   .- -..   .-.. .. - --- .-. .- "
				+ "  - --- .-. --.- ..- . -. -   .--. . .-.   -.-. --- -. ..- -... .. .-  "
				+ " -. --- ... - .-. .-   .--. . .-.   .. -. -.-. . .--. - --- ...  "
				+ " .... -.-- -- . -. .- . --- ...   .- .-.. .. --.- ..- .- --   -. --- -. "
				+ "  . -. .. --   -.. --- -. . -.-.   ..-. . ..- --. .. .- -   - .-. .. ... - .. --.- ..- . "
				+ "  -. .. -... ....   .. -. - . --. . .-.   -- .- --. -. .-   .--- ..- ... - ---  "
				+ " . .-.. . -- . -. - ..- --   . ..-   .-.. .- -.-. .. -. .. .-   ...- . .-..  "
				+ " .. .- -.-. ..- .-.. .. ...   --.- ..- .. ...   . .-.. .. -   -- --- .-. -... ..  "
				+ " .- -   ... .- .--. .. . -.   -- --- .-. -... ..   . --. . -   .. .--. ... ..- --  "
				+ " -.-. --- -. --. ..- .   -. .. ... .-..   -... .-.. .- -. -.. .. -   -- --- "
				+ ".-.. .-.. .. ...   ... . -..   .--- ..- ... - ---   .-.. .. -... . .-. ---  "
				+ " -.-. --- -- -- --- -.. ---   .. -.   ...- . .... .. -.-. ..- .-.. .-   .- "
				+ "  .--. .-. . - .. ..- --   ..-. .- -.-. .. .-.. .. ... .. ...   -. . --.- ..- . "
				+ "  -- .- . -.-. . -. .- ...   .. .- -.-. ..- .-.. .. ...   .-.. .- -.-. ..- ... "
				+ "  .. -..   . .-.. .. -   -. ..- .-.. .-.. .-   ..-. .- -.-. .. .-.. .. ... ..  "
				+ " .- .-.. .. --.- ..- .- --   . .-. .- -   ...- --- .-.. ..- - .--. .- - "
				+ "  ...- . ... - .. -... ..- .-.. ..- --   .-.. .. -... . .-. ---   .- -. - .  "
				+ " ...- . -. . -. .- - .. ...   --.- ..- .. ...   ...- .. ...- . .-. .-. .- "
				+ "  .- -.-.   ... --- -.. .- .-.. . ...   . --. . -   -. .. ... .-..  "
				+ " -. ..- .-.. .-.. .-   --- .-. -. .- .-. .   ...- ..- .-.. .--. ..- - .- - .  "
				+ " -. ..- .-.. .-.. .-   -. ..- .-.. .-.. .- --   --.- ..- .. ...   -- ..  " + " ... . -..   . ..-   -.. ..- ..";

		ProcessableGraph morseStateMachine = buildMorseStateMachine();

		// Test character T
		{
			MorseContext tempContext = new MorseContext();
			// TODO space at end
			tempContext.morseTextCoded = morseSequence + " "; // tailing space

			try {

				morseStateMachine.run(1, tempContext);

				String decoded = tempContext.morseTextDecoded.toString().trim();

				assertEquals(text.toUpperCase(), decoded);

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

	}

	/**
	 * test the performance with a random string that must be decoded.
	 */
	public void testPerformance() {

		final int sizeOfDecodedString = 50000;

		String info = "\n\n\n****\nstarting performance test testPerformance() with sizeOfDecodedString=" + sizeOfDecodedString
				+ ". this might take some seconds...";
		trace(info);

		StringBuffer clearText = new StringBuffer();
		StringBuffer morseSequence = new StringBuffer();

		// first generate random string
		Random rnd = new Random(System.currentTimeMillis());
		String[][] morseCodes = morseCodes();
		for (int i = 0; i < sizeOfDecodedString; i++) {

			int p = Math.abs(rnd.nextInt() % morseCodes.length);
			String character = morseCodes[p][0];
			String morse = morseCodes[p][1];

			// from time to time overrule with a space
			if (i % 20 == 0) {
				character = " ";
				morse = " ";
			}

			clearText.append(character);
			morseSequence.append(morse);
			morseSequence.append(" ");
		}

		ProcessableGraph morseStateMachine = buildMorseStateMachine();

		// switch tracing off!
		TRACE_TO_CONSOLE = false;
		morseStateMachine.getGraphListeners().clear();
		StringBuffer message = new StringBuffer();

		{
			MorseContext tempContext = new MorseContext();
			tempContext.morseTextCoded = morseSequence.toString() + " "; // tailing
			// space

			try {

				long t1 = System.currentTimeMillis();
				morseStateMachine.run(1, tempContext);
				long t2 = System.currentTimeMillis();

				String decoded = tempContext.morseTextDecoded.toString().trim();

				assertEquals(clearText.toString().toUpperCase().trim(), decoded);

				long dt = t2 - t1;
				message.append("\n\n\n*******************\nPerformanceTest testPerformance()");
				//message.append("\ntranslated text=" + clearText.toString());
				message.append("\nsizeClearText=");
				message.append(clearText.toString().length());
				message.append("\nsizeMorseText=");
				int sizeMorseText = morseSequence.toString().length();
				message.append(sizeMorseText);
				message.append("\ntime[ms]=");
				message.append(dt);
				message.append("\nstateTransitions/s is about ");
				// actually little more since we have more states then more code
				// characters
				if (dt != 0) {
					message.append(1000 * sizeMorseText / dt);
				} else {
					message.append("timeToShort dt=0");
				}

			} catch (NoMatchingTransitionConditionFoundException e) {
				fail();
			} catch (MaximumIterationsReachedException e) {
				fail();
			}
		}

		// switch tracing on again
		TRACE_TO_CONSOLE = true;

		trace(message.toString());
	}

	private String[][] morseCodes() {

		String[][] morseCodes = new String[26][2];
		int index = 0;

		morseCodes[index][0] = "A";
		morseCodes[index++][1] = ".-";

		morseCodes[index][0] = "B";
		morseCodes[index++][1] = "-...";

		morseCodes[index][0] = "C";
		morseCodes[index++][1] = "-.-.";

		morseCodes[index][0] = "D";
		morseCodes[index++][1] = "-..";

		morseCodes[index][0] = "E";
		morseCodes[index++][1] = ".";

		morseCodes[index][0] = "F";
		morseCodes[index++][1] = "..-.";

		morseCodes[index][0] = "G";
		morseCodes[index++][1] = "--.";

		morseCodes[index][0] = "H";
		morseCodes[index++][1] = "....";

		morseCodes[index][0] = "I";
		morseCodes[index++][1] = "..";

		morseCodes[index][0] = "J";
		morseCodes[index++][1] = ".---";

		morseCodes[index][0] = "K";
		morseCodes[index++][1] = "-.-";

		morseCodes[index][0] = "L";
		morseCodes[index++][1] = ".-..";

		morseCodes[index][0] = "M";
		morseCodes[index++][1] = "--";

		morseCodes[index][0] = "N";
		morseCodes[index++][1] = "-.";

		morseCodes[index][0] = "O";
		morseCodes[index++][1] = "---";

		morseCodes[index][0] = "P";
		morseCodes[index++][1] = ".--.";

		morseCodes[index][0] = "Q";
		morseCodes[index++][1] = "--.-";

		morseCodes[index][0] = "R";
		morseCodes[index++][1] = ".-.";

		morseCodes[index][0] = "S";
		morseCodes[index++][1] = "...";

		morseCodes[index][0] = "T";
		morseCodes[index++][1] = "-";

		morseCodes[index][0] = "U";
		morseCodes[index++][1] = "..-";

		morseCodes[index][0] = "V";
		morseCodes[index++][1] = "...-";

		morseCodes[index][0] = "W";
		morseCodes[index++][1] = ".--";

		morseCodes[index][0] = "X";
		morseCodes[index++][1] = "-..-";

		morseCodes[index][0] = "Y";
		morseCodes[index++][1] = "-.--";

		morseCodes[index][0] = "Z";
		morseCodes[index++][1] = "--..";
		return morseCodes;
	}

	private ProcessableGraph buildMorseStateMachine() {
		ProcessableGraph graph = new ProcessableGraph();
		graph.registerGraphListener(new TracingGraphListener(System.out));

		Nxt nxtAction = new Nxt();

		// special states
		ProcessableState enterState = new ProcessableState(1, "Enter", new EmptyAction(), false); // start
		ProcessableState spaceState = new ProcessableState(2, "Space", nxtAction, false, new MorseMetaInformation(" "));
		ProcessableState printState = new ProcessableState(3, "Space", new Print(), false);
		ProcessableState outputState = new ProcessableState(4, "Output", new Output(), true); // end

		// character states
		ProcessableState char10 = new ProcessableState(10, "char", nxtAction, false, new MorseMetaInformation("E"));
		ProcessableState char11 = new ProcessableState(11, "char", nxtAction, false, new MorseMetaInformation("T"));
		ProcessableState char12 = new ProcessableState(12, "char", nxtAction, false, new MorseMetaInformation("I"));
		ProcessableState char13 = new ProcessableState(13, "char", nxtAction, false, new MorseMetaInformation("A"));
		ProcessableState char14 = new ProcessableState(14, "char", nxtAction, false, new MorseMetaInformation("N"));
		ProcessableState char15 = new ProcessableState(15, "char", nxtAction, false, new MorseMetaInformation("M"));
		ProcessableState char16 = new ProcessableState(16, "char", nxtAction, false, new MorseMetaInformation("S"));
		ProcessableState char17 = new ProcessableState(17, "char", nxtAction, false, new MorseMetaInformation("U"));
		ProcessableState char18 = new ProcessableState(18, "char", nxtAction, false, new MorseMetaInformation("R"));
		ProcessableState char19 = new ProcessableState(19, "char", nxtAction, false, new MorseMetaInformation("W"));
		ProcessableState char20 = new ProcessableState(20, "char", nxtAction, false, new MorseMetaInformation("D"));
		ProcessableState char21 = new ProcessableState(21, "char", nxtAction, false, new MorseMetaInformation("K"));
		ProcessableState char22 = new ProcessableState(22, "char", nxtAction, false, new MorseMetaInformation("G"));
		ProcessableState char23 = new ProcessableState(23, "char", nxtAction, false, new MorseMetaInformation("O"));
		ProcessableState char24 = new ProcessableState(24, "char", nxtAction, false, new MorseMetaInformation("H"));
		ProcessableState char25 = new ProcessableState(25, "char", nxtAction, false, new MorseMetaInformation("V"));
		ProcessableState char26 = new ProcessableState(26, "char", nxtAction, false, new MorseMetaInformation("F"));
		ProcessableState char27 = new ProcessableState(27, "char", nxtAction, false, new MorseMetaInformation("L"));
		ProcessableState char28 = new ProcessableState(28, "char", nxtAction, false, new MorseMetaInformation("P"));
		ProcessableState char29 = new ProcessableState(29, "char", nxtAction, false, new MorseMetaInformation("J"));
		ProcessableState char30 = new ProcessableState(30, "char", nxtAction, false, new MorseMetaInformation("B"));
		ProcessableState char31 = new ProcessableState(31, "char", nxtAction, false, new MorseMetaInformation("X"));
		ProcessableState char32 = new ProcessableState(32, "char", nxtAction, false, new MorseMetaInformation("C"));
		ProcessableState char33 = new ProcessableState(33, "char", nxtAction, false, new MorseMetaInformation("Y"));
		ProcessableState char34 = new ProcessableState(34, "char", nxtAction, false, new MorseMetaInformation("Z"));
		ProcessableState char35 = new ProcessableState(35, "char", nxtAction, false, new MorseMetaInformation("Q"));

		// starting from enterState
		graph.addValidTransition(enterState, char10, new CharacterCondition("dot"));
		graph.addValidTransition(enterState, char11, new CharacterCondition("dash"));
		graph.addValidTransition(enterState, outputState, new CharacterCondition("end"));
		graph.addValidTransition(enterState, spaceState, new CharacterCondition("space"));

		// starting from spaceState
		graph.addValidTransition(spaceState, printState, new TrueCondition());

		// starting from printState
		graph.addValidTransition(printState, enterState, new TrueCondition());

		// starting from char10
		graph.addValidTransition(char10, char12, new CharacterCondition("dot"));
		graph.addValidTransition(char10, char13, new CharacterCondition("dash"));
		graph.addValidTransition(char10, printState, new CharacterCondition("space"));

		// starting from char11
		graph.addValidTransition(char11, char14, new CharacterCondition("dot"));
		graph.addValidTransition(char11, char15, new CharacterCondition("dash"));
		graph.addValidTransition(char11, printState, new CharacterCondition("space"));

		// starting from char12
		graph.addValidTransition(char12, char16, new CharacterCondition("dot"));
		graph.addValidTransition(char12, char17, new CharacterCondition("dash"));
		graph.addValidTransition(char12, printState, new CharacterCondition("space"));

		// starting from char13
		graph.addValidTransition(char13, char18, new CharacterCondition("dot"));
		graph.addValidTransition(char13, char19, new CharacterCondition("dash"));
		graph.addValidTransition(char13, printState, new CharacterCondition("space"));

		// starting from char14
		graph.addValidTransition(char14, char20, new CharacterCondition("dot"));
		graph.addValidTransition(char14, char21, new CharacterCondition("dash"));
		graph.addValidTransition(char14, printState, new CharacterCondition("space"));

		// starting from char15
		graph.addValidTransition(char15, char22, new CharacterCondition("dot"));
		graph.addValidTransition(char15, char23, new CharacterCondition("dash"));
		graph.addValidTransition(char15, printState, new CharacterCondition("space"));

		// starting from char16
		graph.addValidTransition(char16, char24, new CharacterCondition("dot"));
		graph.addValidTransition(char16, char25, new CharacterCondition("dash"));
		graph.addValidTransition(char16, printState, new CharacterCondition("space"));

		// starting from char17
		graph.addValidTransition(char17, char26, new CharacterCondition("dot"));
		graph.addValidTransition(char17, printState, new CharacterCondition("space"));

		// starting from char18
		graph.addValidTransition(char18, char27, new CharacterCondition("dot"));
		graph.addValidTransition(char18, printState, new CharacterCondition("space"));

		// starting from char19
		graph.addValidTransition(char19, char28, new CharacterCondition("dot"));
		graph.addValidTransition(char19, char29, new CharacterCondition("dash"));
		graph.addValidTransition(char19, printState, new CharacterCondition("space"));

		// starting from char20
		graph.addValidTransition(char20, char30, new CharacterCondition("dot"));
		graph.addValidTransition(char20, char31, new CharacterCondition("dash"));
		graph.addValidTransition(char20, printState, new CharacterCondition("space"));

		// starting from char21
		graph.addValidTransition(char21, char32, new CharacterCondition("dot"));
		graph.addValidTransition(char21, char33, new CharacterCondition("dash"));
		graph.addValidTransition(char21, printState, new CharacterCondition("space"));

		// starting from char22
		graph.addValidTransition(char22, char34, new CharacterCondition("dot"));
		graph.addValidTransition(char22, char35, new CharacterCondition("dash"));
		graph.addValidTransition(char22, printState, new CharacterCondition("space"));

		// starting from char23
		graph.addValidTransition(char23, printState, new CharacterCondition("space"));

		// starting from char24
		graph.addValidTransition(char24, printState, new CharacterCondition("space"));

		// starting from char25
		graph.addValidTransition(char25, printState, new CharacterCondition("space"));

		// starting from char26
		graph.addValidTransition(char26, printState, new CharacterCondition("space"));

		// starting from char27
		graph.addValidTransition(char27, printState, new CharacterCondition("space"));

		// starting from char28
		graph.addValidTransition(char28, printState, new CharacterCondition("space"));

		// starting from char29
		graph.addValidTransition(char29, printState, new CharacterCondition("space"));

		// starting from char30
		graph.addValidTransition(char30, printState, new CharacterCondition("space"));

		// starting from char31
		graph.addValidTransition(char31, printState, new CharacterCondition("space"));

		// starting from char32
		graph.addValidTransition(char32, printState, new CharacterCondition("space"));

		// starting from char33
		graph.addValidTransition(char33, printState, new CharacterCondition("space"));

		// starting from char34
		graph.addValidTransition(char34, printState, new CharacterCondition("space"));

		// starting from char35
		graph.addValidTransition(char35, printState, new CharacterCondition("space"));
		return graph;
	}

	private void consoleNextTest() {
		trace("\n\n-------------------------");
	}

	private static void trace(String msg) {
		if (TRACE_TO_CONSOLE) {
			System.out.println("\t " + msg);
		}

	}

	/**
	 * Prints the last MetaInformation which stores the character.
	 *
	 * @author pheusser
	 */
	private static class Print implements Action {

		private MorseContext morseContext = null;

		private List lastVisitedStatesHistory = null;

		public void execute(ProcessableState currentState, StateContext stateContext, Condition previousCriteria,
				List lastVisitedStatesHistorys) {
			this.morseContext = (MorseContext) stateContext;
			this.lastVisitedStatesHistory = lastVisitedStatesHistorys;

			if (lastVisitedStatesHistory.size() < 2) {
				throw new RuntimeException("could not retrieve the previous state.");
			}
			ProcessableState previousState = (ProcessableState) lastVisitedStatesHistory.get(1);

			MorseMetaInformation previousMorseMetaInfo = (MorseMetaInformation) previousState.getStateMetaInformation();

			// append it to our decoded text
			String decodedChar = previousMorseMetaInfo.character;
			trace("appended decoded char=" + previousMorseMetaInfo.character);
			morseContext.morseTextDecoded.append(decodedChar);

			// move one index forward
			morseContext.morseTextCodedPosition++;

		}

	}

	/**
	 * Moves one character forward in the morse coded input text.
	 *
	 * @author pheusser
	 */
	private static class Nxt implements Action {

		private MorseContext morseContext = null;

		public void execute(ProcessableState currentState, StateContext stateContext, Condition previousCriteria,
				List lastVisitedStatesHistorys) {
			this.morseContext = (MorseContext) stateContext;

			morseContext.morseTextCodedPosition++;

		}

	}

	/**
	 * Write the decoded String out to the console.
	 *
	 * @author pheusser
	 */
	private static class Output implements Action {

		private MorseContext morseContext = null;

		public void execute(ProcessableState currentState, StateContext stateContext, Condition previousCriteria,
				List lastVisitedStatesHistorys) {
			this.morseContext = (MorseContext) stateContext;

			trace("\n**** Output-Action: decoded text=" + morseContext.morseTextDecoded.toString());

		}

	}

	private static class MorseContext implements StateContext {

		protected String morseTextCoded = null;

		protected int morseTextCodedPosition = 0;

		protected StringBuffer morseTextDecoded = new StringBuffer();

		public String toString() {
			return "MorseContext: morseTextCodedPosition=" + morseTextCodedPosition;
		}

	}

	private static class CharacterCondition implements Condition {

		private static final String VERBAL_DASH = "dash";

		private static final String VERBAL_DOT = "dot";

		private static final String VERBAL_SPACE = "space";

		private static final String VERBAL_END = "end";

		private static final String MORSE_SYMBOL_DOT = ".";

		private static final String MORSE_SYMBOL_DASH = "-";

		private static final String MORSE_SYMBOL_SPACE = " ";

		/**
		 * can be VERBAL_DASH, VERBAL_DOT, VERBAL_SPACE or VERBAL_END
		 */
		private String verbalDescription = null;

		/**
		 * @param simpleCriteria
		 */
		public CharacterCondition(String verbalDescription) {
			super();
			this.verbalDescription = verbalDescription;
		}

		public String traceInfo() {
			return "verbalDescription=" + verbalDescription;
		}

		/**
		 * @see com.x8ing.lsm4j.Condition#conditionTrue(com.x8ing.lsm4j.StateContext)
		 */
		public boolean conditionTrue(StateContext currentContext) {

			MorseContext morseContext = (MorseContext) currentContext;

			int currentPos = morseContext.morseTextCodedPosition;

			String currentVerbalDesc = null;
			String currentMorseSymbol = null;
			// translate the current context into a verbal condition
			if (currentPos >= morseContext.morseTextCoded.length()) {
				currentVerbalDesc = VERBAL_END;
			} else {

				// read current morse symbol out of coded text
				currentMorseSymbol = morseContext.morseTextCoded.substring(currentPos, currentPos + 1);

				// figure out symbol
				if (MORSE_SYMBOL_DASH.equals(currentMorseSymbol)) {
					currentVerbalDesc = VERBAL_DASH;
				} else if (MORSE_SYMBOL_DOT.equals(currentMorseSymbol)) {
					currentVerbalDesc = VERBAL_DOT;
				} else if (MORSE_SYMBOL_SPACE.equals(currentMorseSymbol)) {
					currentVerbalDesc = VERBAL_SPACE;
				}
			}

			// check condition
			boolean match = verbalDescription.equals(currentVerbalDesc);

			if (match) {
				trace("we got a conditionMatch. currentPos=" + currentPos + " currentMorseSymbol=" + currentMorseSymbol
						+ " translated into verbalDesc=" + currentVerbalDesc);
			}

			return match;
		}
	}

	/**
	 * Will be attached to a state and contains the character that belongs to the current position in the decoding
	 * graph.
	 */
	private class MorseMetaInformation implements StateMetaInformation {

		protected String character = null;

		public MorseMetaInformation(String character) {
			super();
			this.character = character;
		}

	}

}
