/*
 * Created on May 19, 2007
 */
package com.x8ing.lsm4j.std;

import java.io.PrintStream;

import com.x8ing.lsm4j.Condition;
import com.x8ing.lsm4j.GraphListener;
import com.x8ing.lsm4j.StateContext;
import com.x8ing.lsm4j.state.ProcessableState;

/**
 * A default listener implementation which just traces out some information to the console or a specified PrintStream.
 *
 * @author Patrick Heusser
 */
public class TracingGraphListener implements GraphListener {

	private PrintStream output = null;

	public TracingGraphListener(PrintStream output) {
		this.output = output;
	}

	public TracingGraphListener() {
		this.output = System.out;
	}

	public void changedState(ProcessableState previousState, ProcessableState currentState, Condition previousCondition,
			Condition currentCondition, long loop, StateContext currentStateContext) {

		startTrace();

		output.print("changedState: ");

		output.print("previousState");
		appendState(previousState);
		output.print("currentState");
		appendState(currentState);

		output.print(" previousCondition");
		appendCondition(previousCondition);

		output.print(" currentCondition");
		appendCondition(currentCondition);

		output.print("\n");

	}

	private void appendCondition(Condition condition) {

		output.print(" {");
		if (condition != null) {
			output.print(condition.traceInfo());
		} else {
			output.print("null");
		}
		output.print("} ");
	}

	private void appendState(ProcessableState state) {

		output.print(" {");

		if (state != null) {

			output.print(state.getUniqueIDString());
			output.print(",");
			output.print(state.getDescription());

		} else {

			output.print("null");
		}

		output.print("} ");

	}

	private void startTrace() {
		output.print("[TracingGraphListener] ");
	}

	public void startProcessingState(ProcessableState previousState, Condition previosCondition, long loop,
			StateContext currentStateContext) {

		startTrace();

		output.print("startProcessingState: ");
		appendState(previousState);
		output.print("\n");

	}

	public void foundEndState(ProcessableState endState, long loop, StateContext currentStateContext) {

		startTrace();

		output.print("foundEndState: id=");
		output.print(endState.getUniqueIDString());
		output.print("\n");

	}

}
